VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMP3"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'This file contains a class for detecting MP3 files, as well as
'learning information about the file, and parsing any ID3 tags
'found within the file.  You can also write ID3v1.1 and ID3v2.3
'tags back to the file.

'mp3file - The main class, mp3file, contains all of the functions
'to extract data from the MP3 file.

Option Explicit

Public strMP3Filename As String
Public lngFileSize As Long
Public blnIsMP3 As Boolean
Public strMPEGVersion As String
Public strLayer As String
Public blnProtected As Boolean
Public strSampleRate As String
Public blnPrivateBit As Boolean
Public strChannelMode As String
Public strIntensityStereoBands As String
Public strIntensityStereo As String
Public strMSStereo As String
Public blnCopyright As Boolean
Public blnOriginal As Boolean
Public blnIsVBR As Boolean
Public strID3v1Version As String
Public strTitle As String
Public strArtist As String
Public strAlbum As String
Public strSongYear As String
Public strComment As String
Public strTrack As String
Public lngGenreID As Long
Public strID3v2Version As String
Public lngID3v2Size As Long
Public blnID3v2Footer As Boolean
Public strUFIDOwner As String
Public strUFIDIdentifier As String
Public strTALB As String
Public strTBPM As String
Public strTCOM As String
Public strTCOP As String
Public strTDAT As String
Public strTDEN As String
Public strTDLY As String
Public strTDOR As String
Public strTDRC As String
Public strTDRL As String
Public strTDTG As String
Public strTENC As String
Public strTEXT As String
Public strTFLT As String
Public strTIME As String
Public strTIT1 As String
Public strTIT2 As String
Public strTIT3 As String
Public strTKEY As String
Public strTLAN As String
Public strTLEN As String
Public strTMED As String
Public strTOAL As String
Public strTOFN As String
Public strTOLY As String
Public strTOPE As String
Public strTORY As String
Public strTOWN As String
Public strTPE2 As String
Public strTPE3 As String
Public strTPE4 As String
Public strTPOS As String
Public strTPRO As String
Public strTPUB As String
Public strTRCK As String
Public strTRDA As String
Public strTRSN As String
Public strTRSO As String
Public strTSIZ As String
Public strTSOA As String
Public strTSOP As String
Public strTSOT As String
Public strTSRC As String
Public strTSSE As String
Public strTSST As String
Public strTYER As String
Public strTXXX As String
Public strWCOM As String
Public strWCOP As String
Public strWOAF As String
Public strWOAR As String
Public strWOAS As String
Public strWORS As String
Public strWPAY As String
Public strWPUB As String
Public strWXXX As String
Public strCOMM As String

Private lngTotalBitrate As Long
Private lngNumFrames As Long
Private blnPadding As Boolean
Private lngReadStarts() As Long
Private strVBRIHeader As String
Private strVBRICheck As String
Private lngVBRIStart As Long
Private strVBRIVersion As String
Private strVBRIDelay As String
Private strVBRIQuality As String
Private lngVBRILength As Long
Private lngInitialReadStart As Long
Private intInitialBitrate As Integer
Private intInitialSkipped As Integer
Private strTCON() As String
Private strTIPL() As String
Private strTMCL() As String
Private strTMOO() As String
Private strTPE1() As String

'AvgBitrate - Get the average bitrate of the MP3 file.
Public Property Get AvgBitrate() As Long
    AvgBitrate = CLng((lngTotalBitrate / lngNumFrames) * 1000)
End Property

'SongLength - Get length of the song in seconds
Public Property Get SongLength() As Double
    SongLength = 0.026 * lngNumFrames
End Property

'TIPL - Get the contents of the strTIPL array
Public Property Get TIPL(intIndex As Integer) As String
    TIPL = strTIPL(intIndex)
End Property

'TMCL - Get the contents of the strTMCL array
Public Property Get TMCL(intIndex As Integer) As String
    TMCL = strTMCL(intIndex)
End Property

'TMOO - Get the contents of the strTMOO array
Public Property Get TMOO(intIndex As Integer) As String
    TMOO = strTMOO(intIndex)
End Property

'TPE1 - Get the contents of the strTPE1 array
Public Property Get TPE1(intIndex As Integer) As String
    TPE1 = strTPE1(intIndex)
End Property

'Initialization
Private Sub Class_Initialize()
    blnIsMP3 = False
    strID3v1Version = "none"
    strID3v2Version = "none"
    lngGenreID = -1
End Sub

'LoadMP3 - Loads an MP3 file passed as a path represented as a string.  Returns True if file passed is an MP3, False if not.
Public Function LoadMP3(strFilename) As Boolean
    Dim fso As New FileSystemObject
    Dim f As File
    Dim ts As TextStream
    Dim lngReadStart As Long, lngTagSize As Long, lngExtHeaderSize As Long, lngPaddingSize As Long, lngDecompressedSize As Long, lngDataLength As Long
    Dim blnFoundHeader As Boolean, blnUnsynched As Boolean, blnCompression As Boolean, blnExtHeaderFound As Boolean, blnCRCPresent As Boolean, blnTagAlterPreserve As Boolean, blnFileAlterPreserve As Boolean, blnMP3ReadOnly As Boolean, blnEncryption As Boolean, blnGrouping As Boolean, blnUpdatedTag As Boolean, blnRestrictions As Boolean, blnFrameUnsynched As Boolean, blnDataLengthID As Boolean, blnSuspicious As Boolean, blnCorrectHeader As Boolean
    Dim strReadCheck As String, strID3v2Tag As String, strTagData As String, strExtHeader As String, strTagSizeRestrict As String, strTextEncodingRestrict As String, strTextFieldsSizeRestrict As String, strImageEncodingRestrict As String, strImageSizeRestrict As String, strTOPEData As String, strTOLYData As String, strTMCLData As String, strTIPLData As String, strTMOOData As String, strTPROData As String, strTDENData As String, strTDORData As String, strTDRCData As String, strTDRLData As String, strTDTGData As String, strFooterData As String, strReadBin As String, strID3v1Check As String
    Dim intID3Flags As Integer, intExtHeaderFlags As Integer, intTagStatusFlags As Integer, intTagEncodingFlags As Integer, intEncryptionType As Integer, intGroupID As Integer, intRestrictionFlag As Integer, intBitrate As Integer, intAmountToSkip As Integer, intSkipped As Integer
    Dim dblCRC As Double

    blnIsMP3 = True
    LoadMP3 = True

    'Get filesize.
    On Error Resume Next
    Set f = fso.GetFile(strFilename)
    On Error GoTo 0
    If Not IsObject(f) Then
        blnIsMP3 = False
        LoadMP3 = False
        GoTo EndFunction
    End If
    lngFileSize = f.Size
    Set f = Nothing

    'First step, ID3v2 tag, if any, and then find 11 bit synchronization.
    lngReadStart = -1
    blnFoundHeader = False
    Do
        lngReadStart = lngReadStart + 1
        Set f = Nothing
        Set ts = fso.OpenTextFile(strFilename, 1)
        If lngReadStart > 0 Then ts.Skip lngReadStart
        strReadCheck = ts.Read(4)

        If Len(strReadCheck) < 4 Then
            blnIsMP3 = False
            LoadMP3 = False
            GoTo EndFunction
        End If

        If strReadCheck = "OggS" Then
            blnIsMP3 = False
            LoadMP3 = False
            GoTo EndFunction
        End If

        'ID3v2 tag found.  Parse and skip it.
        If lngReadStart = 0 And Mid(strReadCheck, 1, 3) = "ID3" Then
            Select Case Asc(Mid(strReadCheck, 4, 1))
                Case 2 'ID3v2.2
                    strReadCheck = strReadCheck & ts.Read(6)
                    strID3v2Version = CStr(Asc(Mid(strReadCheck, 4, 1))) & "." & CStr(Asc(Mid(strReadCheck, 5, 1)))
                    intID3Flags = Asc(Mid(strReadCheck, 6, 1))
                    If intID3Flags >= 128 Then
                        intID3Flags = intID3Flags - 128
                        blnUnsynched = True
                    Else
                        blnUnsynched = False
                    End If
                    If intID3Flags >= 64 Then
                        intID3Flags = intID3Flags - 64
                        blnCompression = True
                    Else
                        blnCompression = False
                    End If
                    lngID3v2Size = 2097152 * Asc(Mid(strReadCheck, 7, 1)) + 16384 * Asc(Mid(strReadCheck, 8, 1)) + 128 * Asc(Mid(strReadCheck, 9, 1)) + Asc(Mid(strReadCheck, 10, 1))
                    If lngID3v2Size = 0 Then
                        strID3v2Version = "none"
                    Else
                        strID3v2Tag = ts.Read(lngID3v2Size)
                        If blnUnsynched Then strID3v2Tag = Replace(strID3v2Tag, Chr(255) & Chr(0), Chr(255))

                        Do While Len(strID3v2Tag) > 6
                            If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strID3v2Tag, 1, 1)) = 0 Then Exit Do
                            lngTagSize = 65536 * Asc(Mid(strID3v2Tag, 4, 1)) + 256 * Asc(Mid(strID3v2Tag, 5, 1)) + Asc(Mid(strID3v2Tag, 6, 1))
                            If lngTagSize = 0 Then Exit Do
                            strTagData = Mid(strID3v2Tag, 7, lngTagSize)
                            ProcessID3v2_2Tag strID3v2Tag, strTagData
                            lngTagSize = lngTagSize + 6
                            strID3v2Tag = Mid(strID3v2Tag, lngTagSize + 1, Len(strID3v2Tag) - lngTagSize)
                        Loop
                        If Len(strCOMM) > 0 Then
                            strCOMM = Mid(strCOMM, 1, Len(strCOMM) - 1)
                            strComment = Replace(strCOMM, Chr(0), vbCrLf)
                        End If
                    End If
                Case 3 'ID3v2.3
                    strReadCheck = strReadCheck & ts.Read(6)
                    strID3v2Version = CStr(Asc(Mid(strReadCheck, 4, 1))) & "." & CStr(Asc(Mid(strReadCheck, 5, 1)))
                    intID3Flags = Asc(Mid(strReadCheck, 6, 1))
                    lngID3v2Size = 2097152 * Asc(Mid(strReadCheck, 7, 1)) + 16384 * Asc(Mid(strReadCheck, 8, 1)) + 128 * Asc(Mid(strReadCheck, 9, 1)) + Asc(Mid(strReadCheck, 10, 1))
                    If intID3Flags >= 128 Then
                        intID3Flags = intID3Flags - 128
                        blnUnsynched = True
                    Else
                        blnUnsynched = False
                    End If
                    If intID3Flags >= 64 Then
                        intID3Flags = intID3Flags - 64
                        blnExtHeaderFound = True
                    Else
                        blnExtHeaderFound = False
                    End If
                    If intID3Flags >= 32 Or lngID3v2Size = 0 Then
                        strID3v2Version = "none"
                    Else
                        strID3v2Tag = ts.Read(lngID3v2Size)
                        If blnUnsynched Then strID3v2Tag = Replace(strID3v2Tag, Chr(255) & Chr(0), Chr(255))

                        If blnExtHeaderFound Then
                            lngExtHeaderSize = 16777216 * Asc(Mid(strID3v2Tag, 1, 1)) + 65536 * Asc(Mid(strID3v2Tag, 2, 1)) + 256 * Asc(Mid(strID3v2Tag, 3, 1)) + Asc(Mid(strID3v2Tag, 4, 1))
                            strExtHeader = Mid(strID3v2Tag, 5, lngExtHeaderSize)
                            strID3v2Tag = Mid(strID3v2Tag, 5 + lngExtHeaderSize, Len(strID3v2Tag) - (lngExtHeaderSize + 4))
                            intExtHeaderFlags = Asc(Mid(strExtHeader, 1, 1))
                            If intExtHeaderFlags >= 128 Then
                                blnCRCPresent = True
                            Else
                                blnCRCPresent = False
                            End If
                            lngPaddingSize = 16777216 * Asc(Mid(strExtHeader, 3, 1)) + 65536 * Asc(Mid(strExtHeader, 4, 1)) + 256 * Asc(Mid(strExtHeader, 5, 1)) + Asc(Mid(strExtHeader, 6, 1))
                            If blnCRCPresent Then
                                dblCRC = 16777216# * Asc(Mid(strExtHeader, 7, 1)) + 65536# * Asc(Mid(strExtHeader, 8, 1)) + 256# * Asc(Mid(strExtHeader, 9, 1)) + Asc(Mid(strExtHeader, 10, 1))
                            Else
                                dblCRC = 0
                            End If
                        End If

                        strTXXX = ""
                        strWOAR = ""
                        strWCOM = ""
                        strWXXX = ""
                        strCOMM = ""
                        Do While Len(strID3v2Tag) > 10
                            If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strID3v2Tag, 1, 1)) = 0 Then Exit Do
                            lngTagSize = 16777216 * Asc(Mid(strID3v2Tag, 5, 1)) + 65536 * Asc(Mid(strID3v2Tag, 6, 1)) + 256 * Asc(Mid(strID3v2Tag, 7, 1)) + Asc(Mid(strID3v2Tag, 8, 1))
                            If lngTagSize = 0 Then Exit Do
                            intTagStatusFlags = Asc(Mid(strID3v2Tag, 9, 1))
                            intTagEncodingFlags = Asc(Mid(strID3v2Tag, 10, 1))
                            strTagData = Mid(strID3v2Tag, 11, lngTagSize)
                            If intTagStatusFlags >= 128 Then
                                intTagStatusFlags = intTagStatusFlags - 128
                                blnTagAlterPreserve = True
                            Else
                                blnTagAlterPreserve = False
                            End If
                            If intTagStatusFlags >= 64 Then
                                intTagStatusFlags = intTagStatusFlags - 64
                                blnFileAlterPreserve = True
                            Else
                                blnFileAlterPreserve = False
                            End If
                            If intTagStatusFlags >= 32 Then
                                blnMP3ReadOnly = True
                            Else
                                blnMP3ReadOnly = False
                            End If
                            If intTagEncodingFlags >= 128 Then
                                intTagEncodingFlags = intTagEncodingFlags - 128
                                blnCompression = True
                            Else
                                blnCompression = False
                            End If
                            If intTagEncodingFlags >= 64 Then
                                intTagEncodingFlags = intTagEncodingFlags - 64
                                blnEncryption = True
                            Else
                                blnEncryption = False
                            End If
                            If intTagEncodingFlags >= 32 Then
                                blnGrouping = True
                            Else
                                blnGrouping = False
                            End If
                            If blnCompression Then
                                lngDecompressedSize = 16777216 * Asc(Mid(strTagData, 1, 1)) + 65536 * Asc(Mid(strTagData, 2, 1)) + 256 * Asc(Mid(strTagData, 3, 1)) + Asc(Mid(strTagData, 4, 1))
                                strTagData = Mid(strTagData, 5, Len(strTagData) - 4)
                            Else
                                lngDecompressedSize = 0
                            End If
                            If blnEncryption Then
                                intEncryptionType = Asc(Mid(strTagData, 1, 1))
                                strTagData = Mid(strTagData, 2, Len(strTagData) - 1)
                            Else
                                intEncryptionType = 0
                            End If
                            If blnGrouping Then
                                intGroupID = Asc(Mid(strTagData, 1, 1))
                                strTagData = Mid(strTagData, 2, Len(strTagData) - 1)
                            Else
                                intGroupID = 0
                            End If
                            ProcessID3v2_3Tag strID3v2Tag, strTagData
                            lngTagSize = lngTagSize + 10
                            strID3v2Tag = Mid(strID3v2Tag, lngTagSize + 1, Len(strID3v2Tag) - lngTagSize)
                        Loop
                        If Len(strCOMM) > 0 Then
                            strCOMM = Mid(strCOMM, 1, Len(strCOMM) - 1)
                            strComment = Replace(strCOMM, Chr(0), vbCrLf)
                        End If
                    End If
                Case 4 'ID3v2.4
                    strReadCheck = strReadCheck & ts.Read(6)
                    strID3v2Version = CStr(Asc(Mid(strReadCheck, 4, 1))) & "." & CStr(Asc(Mid(strReadCheck, 5, 1)))
                    intID3Flags = Asc(Mid(strReadCheck, 6, 1))
                    lngID3v2Size = 2097152 * Asc(Mid(strReadCheck, 7, 1)) + 16384 * Asc(Mid(strReadCheck, 8, 1)) + 128 * Asc(Mid(strReadCheck, 9, 1)) + Asc(Mid(strReadCheck, 10, 1))
                    If intID3Flags >= 128 Then
                        intID3Flags = intID3Flags - 128
                        blnUnsynched = True
                    Else
                        blnUnsynched = False
                    End If
                    If intID3Flags >= 64 Then
                        intID3Flags = intID3Flags - 64
                        blnExtHeaderFound = True
                    Else
                        blnExtHeaderFound = False
                    End If
                    If intID3Flags >= 32 Then
                        strID3v2Version = "none"
                    End If
                    If intID3Flags >= 16 Then
                        blnID3v2Footer = True
                    Else
                        blnID3v2Footer = False
                    End If
                    If lngID3v2Size = 0 Or strID3v2Version = "none" Then
                        strID3v2Version = "none"
                    Else
                        strID3v2Tag = ts.Read(lngID3v2Size)
                        If blnUnsynched Then strID3v2Tag = Replace(strID3v2Tag, Chr(255) & Chr(0), Chr(255))

                        If blnExtHeaderFound Then
                            lngExtHeaderSize = 2097152 * Asc(Mid(strID3v2Tag, 1, 1)) + 16384 * Asc(Mid(strID3v2Tag, 2, 1)) + 128 * Asc(Mid(strID3v2Tag, 3, 1)) + Asc(Mid(strID3v2Tag, 4, 1))
                            strExtHeader = Mid(strID3v2Tag, 5, lngExtHeaderSize - 4)
                            strID3v2Tag = Mid(strID3v2Tag, 1 + lngExtHeaderSize, Len(strID3v2Tag) - lngExtHeaderSize)
                            intExtHeaderFlags = Asc(Mid(strExtHeader, 2, 1))
                            If intExtHeaderFlags >= 64 Then
                                intExtHeaderFlags = intExtHeaderFlags - 64
                                blnUpdatedTag = True
                            Else
                                blnUpdatedTag = False
                            End If
                            If intExtHeaderFlags >= 32 Then
                                intExtHeaderFlags = intExtHeaderFlags - 32
                                blnCRCPresent = True
                            Else
                                blnCRCPresent = False
                            End If
                            If intExtHeaderFlags >= 16 Then
                                blnRestrictions = True
                            Else
                                blnRestrictions = False
                            End If
                            If blnCRCPresent Then
                                dblCRC = 268435456 * Asc(Mid(strExtHeader, 3, 1)) + 2097152 * Asc(Mid(strExtHeader, 4, 1)) + 16384 * Asc(Mid(strExtHeader, 5, 1)) + 128 * Asc(Mid(strExtHeader, 6, 1)) + Asc(Mid(strExtHeader, 7, 1))
                            Else
                                dblCRC = 0
                            End If
                            If blnRestrictions Then
                                intRestrictionFlag = Asc(Mid(strExtHeader, Len(strExtHeader), 1))
                                If intRestrictionFlag >= 192 Then
                                    intRestrictionFlag = intRestrictionFlag - 192
                                    strTagSizeRestrict = "32/4"
                                ElseIf intRestrictionFlag >= 128 Then
                                    intRestrictionFlag = intRestrictionFlag - 128
                                    strTagSizeRestrict = "32/40"
                                ElseIf intRestrictionFlag >= 64 Then
                                    intRestrictionFlag = intRestrictionFlag - 64
                                    strTagSizeRestrict = "64/128"
                                Else
                                    strTagSizeRestrict = "128/1024"
                                End If
                                If intRestrictionFlag >= 32 Then
                                    intRestrictionFlag = intRestrictionFlag - 32
                                    strTextEncodingRestrict = "ISO-8859-1/UTF-8"
                                Else
                                    strTextEncodingRestrict = "none"
                                End If
                                If intRestrictionFlag >= 24 Then
                                    intRestrictionFlag = intRestrictionFlag - 24
                                    strTextFieldsSizeRestrict = "30"
                                ElseIf intRestrictionFlag >= 16 Then
                                    intRestrictionFlag = intRestrictionFlag - 16
                                    strTextFieldsSizeRestrict = "128"
                                ElseIf intRestrictionFlag >= 8 Then
                                    intRestrictionFlag = intRestrictionFlag - 8
                                    strTextFieldsSizeRestrict = "1024"
                                Else
                                    strTextFieldsSizeRestrict = "none"
                                End If
                                If intRestrictionFlag >= 4 Then
                                    intRestrictionFlag = intRestrictionFlag - 4
                                    strImageEncodingRestrict = "PNG/JPEG"
                                Else
                                    strImageEncodingRestrict = "none"
                                End If
                                If intRestrictionFlag = 3 Then
                                    strImageSizeRestrict = "=64x64"
                                ElseIf intRestrictionFlag = 2 Then
                                    strImageSizeRestrict = "<=64x64"
                                ElseIf intRestrictionFlag = 1 Then
                                    strImageSizeRestrict = "<=256x256"
                                Else
                                    strImageSizeRestrict = "none"
                                End If
                            Else
                                strTagSizeRestrict = "none"
                                strTextEncodingRestrict = "none"
                                strTextFieldsSizeRestrict = "none"
                                strImageEncodingRestrict = "none"
                                strImageSizeRestrict = "none"
                            End If
                        End If

                        strTXXX = ""
                        strWOAR = ""
                        strWCOM = ""
                        strWXXX = ""
                        strCOMM = ""
                        Do While Len(strID3v2Tag) > 10
                            If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strID3v2Tag, 1, 1)) = 0 Then Exit Do
                            lngTagSize = 2097152 * Asc(Mid(strID3v2Tag, 5, 1)) + 16384 * Asc(Mid(strID3v2Tag, 6, 1)) + 128 * Asc(Mid(strID3v2Tag, 7, 1)) + Asc(Mid(strID3v2Tag, 8, 1))
                            If lngTagSize = 0 Then Exit Do
                            intTagStatusFlags = Asc(Mid(strID3v2Tag, 9, 1))
                            intTagEncodingFlags = Asc(Mid(strID3v2Tag, 10, 1))
                            strTagData = Mid(strID3v2Tag, 11, lngTagSize)
                            If intTagStatusFlags >= 64 Then
                                intTagStatusFlags = intTagStatusFlags - 64
                                blnTagAlterPreserve = True
                            Else
                                blnTagAlterPreserve = False
                            End If
                            If intTagStatusFlags >= 32 Then
                                intTagStatusFlags = intTagStatusFlags - 32
                                blnFileAlterPreserve = True
                            Else
                                blnFileAlterPreserve = False
                            End If
                            If intTagStatusFlags >= 16 Then
                                blnMP3ReadOnly = True
                            Else
                                blnMP3ReadOnly = False
                            End If
                            If intTagEncodingFlags >= 64 Then
                                intTagEncodingFlags = intTagEncodingFlags - 64
                                blnGrouping = True
                            Else
                                blnGrouping = False
                            End If
                            If intTagEncodingFlags >= 8 Then
                                intTagEncodingFlags = intTagEncodingFlags - 8
                                blnCompression = True
                            Else
                                blnCompression = False
                            End If
                            If intTagEncodingFlags >= 4 Then
                                intTagEncodingFlags = intTagEncodingFlags - 4
                                blnEncryption = True
                            Else
                                blnEncryption = False
                            End If
                            If intTagEncodingFlags >= 2 Then
                                intTagEncodingFlags = intTagEncodingFlags - 2
                                blnFrameUnsynched = True
                            Else
                                blnFrameUnsynched = False
                            End If
                            If intTagEncodingFlags >= 1 Then
                                blnDataLengthID = True
                            Else
                                blnDataLengthID = False
                            End If
                            If blnGrouping Then
                                intGroupID = Asc(Mid(strTagData, 1, 1))
                                strTagData = Mid(strTagData, 2, Len(strTagData) - 1)
                            Else
                                intGroupID = 0
                            End If
                            If blnCompression Then
                                lngDecompressedSize = 16777216 * Asc(Mid(strTagData, 1, 1)) + 65536 * Asc(Mid(strTagData, 2, 1)) + 256 * Asc(Mid(strTagData, 3, 1)) + Asc(Mid(strTagData, 4, 1))
                                strTagData = Mid(strTagData, 5, Len(strTagData) - 4)
                            Else
                                lngDecompressedSize = 0
                            End If
                            If blnEncryption Then
                                intEncryptionType = Asc(Mid(strTagData, 1, 1))
                                strTagData = Mid(strTagData, 2, Len(strTagData) - 1)
                            Else
                                intEncryptionType = 0
                            End If
                            If blnDataLengthID Then
                                lngDataLength = 16777216 * Asc(Mid(strTagData, 1, 1)) + 65536 * Asc(Mid(strTagData, 2, 1)) + 256 * Asc(Mid(strTagData, 3, 1)) + Asc(Mid(strTagData, 4, 1))
                                strTagData = Mid(strTagData, 5, Len(strTagData) - 4)
                            Else
                                lngDataLength = 0
                            End If
                            If blnFrameUnsynched And Not blnUnsynched Then
                                strID3v2Tag = Replace(strID3v2Tag, Chr(255) & Chr(0), Chr(255))
                            End If
                            ProcessID3v2_4Tag strID3v2Tag, strTagData
                            lngTagSize = lngTagSize + 10
                            strID3v2Tag = Mid(strID3v2Tag, lngTagSize + 1, Len(strID3v2Tag) - lngTagSize)
                        Loop
                        If Len(strCOMM) > 0 Then
                            strCOMM = Mid(strCOMM, 1, Len(strCOMM) - 1)
                            strComment = Replace(strCOMM, Chr(0), vbCrLf)
                        End If
                        If blnID3v2Footer Then
                            strFooterData = ts.Read(10)
                        End If
                    End If
                Case Else
                    strID3v2Version = "none"
            End Select
            lngReadStart = lngTagSize + 10
        ElseIf Asc(Mid(strReadCheck, 1, 1)) = 255 And Asc(Mid(strReadCheck, 2, 1)) >= 224 And Asc(Mid(strReadCheck, 3, 1)) >= 16 Then
            blnFoundHeader = True
        Else
            If Asc(Mid(strReadCheck, 2, 1)) < 255 Then
                lngReadStart = lngReadStart + 1
                If Asc(Mid(strReadCheck, 3, 1)) < 255 Then
                    lngReadStart = lngReadStart + 1
                    If Asc(Mid(strReadCheck, 4, 1)) < 255 Then
                        lngReadStart = lngReadStart + 1
                    End If
                End If
            End If
        End If
    Loop Until blnFoundHeader = True Or lngReadStart = lngFileSize - 2

    'If header not found, bail.
    If Not blnFoundHeader Then
        blnIsMP3 = False
        LoadMP3 = False
        GoTo EndFunction
    End If

    'Some encoders throw a first byte of 255 in because they don't care of the impact it will have on us programmers.  Check for this.
    Do
        'Convert header to binary.
        strReadBin = ToBin(strReadCheck)

        'Set suspicious flag.
        If Mid(strReadBin, 9, 8) = "11111111" Then
            blnSuspicious = True
        Else
            blnSuspicious = False
        End If

        'Read header.
        intBitrate = ReadHeader(strReadBin)

        If intBitrate = -1 Then
            If blnSuspicious Then
                strReadCheck = Right(strReadCheck, 3) & ts.Read(1)
                lngReadStart = lngReadStart + 1
            Else
                blnIsMP3 = False
                LoadMP3 = False
                GoTo EndFunction
            End If
        Else
            lngTotalBitrate = intBitrate
            lngNumFrames = 1
        End If
    Loop While blnSuspicious

    'Some encoders are buggy and write VBRI data incorrectly.  Detect VBRI data and skip it, it will be rewritten later.
    ts.Skip 32
    strVBRICheck = ts.Read(4)
    If strVBRICheck = "VBRI" Then
        'Read old VBRI information, will be needed later
        strVBRIHeader = strReadCheck
        lngVBRIStart = lngReadStart
        strVBRIVersion = ts.Read(2)
        strVBRIDelay = ts.Read(2)
        strVBRIQuality = ts.Read(2)

        'Skip ahead to find the next header
        lngReadStart = lngReadStart + Int(144000 * intBitrate / CLng(strSampleRate))
        intAmountToSkip = Int(144000 * intBitrate / CLng(strSampleRate)) - 4
        If blnPadding Then
            lngReadStart = lngReadStart + 1
            intAmountToSkip = intAmountToSkip + 1
        End If
        lngVBRILength = intAmountToSkip + 4
        If lngReadStart > 0 Then
            ts.Skip (intAmountToSkip - 42)
            intSkipped = 0
        End If

        'Find the first correct header.
        blnCorrectHeader = False
        strReadCheck = ts.Read(4)
        Do
            'Convert header to binary.
            strReadBin = ToBin(strReadCheck)

            'Read header.
            blnIsMP3 = True
            intBitrate = ReadHeader(strReadBin)

            If intBitrate = -1 Or blnIsMP3 = False Or Not (Asc(Mid(strReadCheck, 1, 1)) = 255 And Asc(Mid(strReadCheck, 2, 1)) >= 224) Then
                strReadCheck = Right(strReadCheck, 3) & ts.Read(1)
                lngReadStart = lngReadStart + 1
            Else
                lngTotalBitrate = intBitrate
                lngNumFrames = 1
                blnCorrectHeader = True
            End If
        Loop While Not blnCorrectHeader
    Else
        intSkipped = 36
        lngVBRIStart = 0
        lngVBRILength = 0
    End If
    ts.Close
    Set ts = Nothing
    lngInitialReadStart = lngReadStart
    intInitialBitrate = intBitrate
    intInitialSkipped = lngReadStart

    'Check for and read ID3v1.0 and ID3v1.1 tags.
    Set ts = fso.OpenTextFile(strFilename, 1)
    ts.Skip (lngFileSize - 128)
    strID3v1Check = ts.Read(128)
    If Mid(strID3v1Check, 1, 3) = "TAG" Then
        If Asc(Mid(strID3v1Check, 126, 1)) = 0 And Asc(Mid(strID3v1Check, 127, 1)) > 0 Then
            strID3v1Version = "1.1"
        Else
            strID3v1Version = "1.0"
        End If
        If Len(NullTruncate(Mid(strID3v1Check, 4, 30))) > Len(strTitle) Then
            strTitle = NullTruncate(Mid(strID3v1Check, 4, 30))
        End If
        If Len(NullTruncate(Mid(strID3v1Check, 34, 30))) > Len(strArtist) Then
            strArtist = NullTruncate(Mid(strID3v1Check, 34, 30))
        End If
        If Len(NullTruncate(Mid(strID3v1Check, 64, 30))) > Len(strAlbum) Then
            strAlbum = NullTruncate(Mid(strID3v1Check, 64, 30))
        End If
        If Len(NullTruncate(Mid(strID3v1Check, 94, 4))) > Len(strSongYear) Then
            strSongYear = NullTruncate(Mid(strID3v1Check, 94, 4))
        End If
        If strID3v1Version = "1.0" Then
            If Len(NullTruncate(Mid(strID3v1Check, 98, 30))) > Len(strComment) Then
                strComment = NullTruncate(Mid(strID3v1Check, 98, 30))
            End If
        Else
            If Len(NullTruncate(Mid(strID3v1Check, 98, 28))) > Len(strComment) Then
                strComment = NullTruncate(Mid(strID3v1Check, 98, 28))
            End If
            If Len(CStr(Asc(Mid(strID3v1Check, 127, 1)))) > Len(strTrack) Then
                strTrack = CStr(Asc(Mid(strID3v1Check, 127, 1)))
            End If
        End If
        If Len(lngGenreID) = 0 Then
            lngGenreID = CStr(Asc(Mid(strID3v1Check, 128, 1)))
            If Not (lngGenreID >= 0 And lngGenreID <= 147) Then
                lngGenreID = ""
            End If
        End If
    End If
    
    strMP3Filename = strFilename
EndFunction:
    On Error Resume Next
    ts.Close
    On Error GoTo 0
    Set ts = Nothing
    Set f = Nothing
    Set fso = Nothing
End Function

'CalculateAverageBitrate - Calculates the average bitrate of the MP3 file.
Public Function CalculateAverageBitrate()
    Dim fso As New FileSystemObject
    Dim ts As TextStream
    Dim lngReadStart As Long
    Dim intBitrate As Integer, intAmountToSkip As Integer, intSkipped As Integer
    Dim strReadCheck As String, strReadBin As String
    
    lngReadStart = lngInitialReadStart
    intBitrate = intInitialBitrate
    intSkipped = intInitialSkipped
    Set ts = fso.OpenTextFile(strMP3Filename)
    If lngReadStart > 0 Then
        ts.Skip lngReadStart
    End If
    
    ReDim Preserve lngReadStarts(1)
    lngReadStarts(1) = lngReadStart
    blnIsVBR = False
    lngReadStart = lngReadStart + Int(144000 * intBitrate / CLng(strSampleRate))
    intAmountToSkip = Int(144000 * intBitrate / CLng(strSampleRate)) - 4
    If blnPadding Then
        lngReadStart = lngReadStart + 1
        intAmountToSkip = intAmountToSkip + 1
    End If

    Do While lngReadStart < lngFileSize
        If lngReadStart > 0 Then
            ts.Skip (intAmountToSkip - intSkipped)
            intSkipped = 0
        End If
        strReadCheck = ts.Read(4)
        If Not (Asc(Mid(strReadCheck, 1, 1)) = 255 And Asc(Mid(strReadCheck, 2, 1)) >= 224) Then
            CalculateAverageBitrate = True
            Exit Do
        End If

        strReadBin = ToBin(strReadCheck)

        intBitrate = GetBitrate(Mid(strReadBin, 17, 4), strMPEGVersion, strLayer)
        If intBitrate = -1 Then
            lngTotalBitrate = 0
            blnIsMP3 = False
            CalculateAverageBitrate = False
            Exit Do
        End If
        If intBitrate > 0 Then
            If lngNumFrames > 0 Then
                If lngTotalBitrate / lngNumFrames <> intBitrate Then blnIsVBR = True
            End If
            lngTotalBitrate = lngTotalBitrate + intBitrate
            lngNumFrames = lngNumFrames + 1
            ReDim Preserve lngReadStarts(lngNumFrames)
            lngReadStarts(lngNumFrames) = lngReadStart
        End If

        If Mid(strReadBin, 23, 1) = "0" Then blnPadding = False Else blnPadding = True

        lngReadStart = lngReadStart + Int(144000 * intBitrate / CLng(strSampleRate))
        intAmountToSkip = Int(144000 * intBitrate / CLng(strSampleRate)) - 4
        If blnPadding Then
            lngReadStart = lngReadStart + 1
            intAmountToSkip = intAmountToSkip + 1
        End If
    Loop
    
    ts.Close
    ts = Nothing
    fso = Nothing
End Function


'RewriteVBRITag - Rewrites VBRI Tags so they are less buggy.
Public Sub RewriteVBRITag()
    If strVBRICheck <> "VBRI" Then Exit Sub
    
    Dim stmBinaryStream As New Stream
    Dim lngThisNumFrames As Long, lngNewFilesize As Long, lngStreamSize As Long, lngMaxOffset As Long, lngNumSeekOffsets As Long, lngOffsetScale As Long, lngOffsetSize As Long
    Dim intOffsetCount As Integer
    Dim strVBRIFrame As String
    Dim lngOffsets() As Long
    Dim bytVBRIHeaderBinary() As Byte, bytReadBinaryFilePreVBRI() As Byte, bytReadBinaryFile() As Byte
    
    lngThisNumFrames = UBound(lngReadStarts)
    lngNewFilesize = lngFileSize - (lngReadStarts(1) - (lngVBRIStart + lngVBRILength))
    lngStreamSize = lngFileSize - lngReadStarts(1)
    If strID3v1Version <> "none" Then
        lngStreamSize = lngStreamSize - 128
    End If
    lngNumSeekOffsets = Int((lngVBRILength - 62) / 2)
    lngOffsetSize = Int(lngThisNumFrames / lngNumSeekOffsets)
    
    ReDim Preserve lngOffsets(lngNumSeekOffsets)
    lngOffsets(1) = lngReadStarts(lngOffsetSize)
    lngMaxOffset = lngReadStarts(lngOffsetSize)
    For intOffsetCount = 2 To lngNumSeekOffsets
        lngOffsets(intOffsetCount) = lngReadStarts(intOffsetCount * lngOffsetSize) - lngReadStarts((intOffsetCount - 1) * lngOffsetSize)
        If lngOffsets(intOffsetCount) > lngMaxOffset Then
            lngMaxOffset = lngOffsets(intOffsetCount)
        End If
    Next
    lngOffsetScale = Int(lngMaxOffset / 65536) + 1
    strVBRIFrame = strVBRIHeader & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & "VBRI" & strVBRIVersion & strVBRIDelay & strVBRIQuality & GetBinaryInt(lngStreamSize) & GetBinaryInt(lngThisNumFrames) & Right(GetBinaryInt(lngNumSeekOffsets), 2) & Right(GetBinaryInt(lngOffsetScale), 2) & Chr(0) & Chr(2) & Right(GetBinaryInt(lngOffsetSize), 2)
    For intOffsetCount = 1 To lngNumSeekOffsets
        strVBRIFrame = strVBRIFrame & Right(GetBinaryInt(Int(lngOffsets(intOffsetCount) / lngOffsetScale)), 2)
    Next
    Do While Len(strVBRIFrame) < lngVBRILength
        strVBRIFrame = strVBRIFrame & Asc(0)
    Loop
    bytVBRIHeaderBinary = Stream_StringToBinary(strVBRIFrame)
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    stmBinaryStream.LoadFromFile strMP3Filename
    If lngVBRIStart > 0 Then
        bytReadBinaryFilePreVBRI = stmBinaryStream.Read(lngVBRIStart)
    End If
    bytReadBinaryFile = stmBinaryStream.Read(lngVBRILength)
    If lngVBRIStart + lngVBRILength < lngReadStarts(1) Then
        bytReadBinaryFile = stmBinaryStream.Read(lngReadStarts(1) - (lngVBRIStart + lngVBRILength))
    End If
    bytReadBinaryFile = stmBinaryStream.Read(lngFileSize - lngVBRILength - lngVBRIStart)
    stmBinaryStream.Close
    Set stmBinaryStream = CreateObject("ADODB.Stream")
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    If lngVBRIStart > 0 Then
        stmBinaryStream.Write bytReadBinaryFilePreVBRI
    End If
    stmBinaryStream.Write bytVBRIHeaderBinary
    stmBinaryStream.Write bytReadBinaryFile
    stmBinaryStream.SaveToFile strMP3Filename, adSaveCreateOverWrite
    stmBinaryStream.Close
    lngFileSize = lngNewFilesize
    strVBRICheck = "Already Rewritten"
    Set stmBinaryStream = Nothing
End Sub

'StripID3v1 - Removes ID3v1 tags from the file, if they exist.
Public Sub StripID3v1()
    Dim fso As New FileSystemObject
    Dim ts As TextStream
    Dim stmBinaryStream As New Stream
    Dim strID3v1Check As String
    Dim bytReadBinaryFile() As Byte

    If strID3v1Version = "none" Then Exit Sub
    Set ts = fso.OpenTextFile(strMP3Filename, 1)
    ts.Skip (lngFileSize - 128)
    strID3v1Check = ts.Read(128)
    ts.Close
    If Mid(strID3v1Check, 1, 3) = "TAG" Then
        stmBinaryStream.Type = adTypeBinary
        stmBinaryStream.Open
        stmBinaryStream.LoadFromFile strMP3Filename
        bytReadBinaryFile = stmBinaryStream.Read(lngFileSize - 128)
        stmBinaryStream.Close
        Set stmBinaryStream = CreateObject("ADODB.Stream")
        stmBinaryStream.Type = adTypeBinary
        stmBinaryStream.Open
        stmBinaryStream.Write bytReadBinaryFile
        stmBinaryStream.SaveToFile strMP3Filename, adSaveCreateOverWrite
        stmBinaryStream.Close
        Set stmBinaryStream = Nothing
        strID3v1Version = "none"
        lngFileSize = lngFileSize - 128
    End If

    Set stmBinaryStream = Nothing
    Set ts = Nothing
    Set fso = Nothing
End Sub

'StripID3v2 - Removes ID3v2 tags from the file, if they exist.
Public Sub StripID3v2()
    Dim fso As New FileSystemObject
    Dim ts As TextStream
    Dim strID3v2Check As String
    Dim stmBinaryStream As New Stream
    Dim intTagIDSize As Integer
    Dim bytReadBinaryFile() As Byte

    If strID3v2Version = "none" Then Exit Sub
    Set ts = fso.OpenTextFile(strMP3Filename, 1)
    strID3v2Check = ts.Read(3)
    ts.Close
    If strID3v2Check = "ID3" Then
        stmBinaryStream.Type = adTypeBinary
        stmBinaryStream.Open
        stmBinaryStream.LoadFromFile strMP3Filename
        If blnID3v2Footer Then
            intTagIDSize = 20
        Else
            intTagIDSize = 10
        End If
        bytReadBinaryFile = stmBinaryStream.Read(lngID3v2Size + intTagIDSize) 'Discard the ID3Tag.
        bytReadBinaryFile = stmBinaryStream.Read(lngFileSize - lngID3v2Size)
        stmBinaryStream.Close
        Set stmBinaryStream = CreateObject("ADODB.Stream")
        stmBinaryStream.Type = adTypeBinary
        stmBinaryStream.Open
        stmBinaryStream.Write bytReadBinaryFile
        stmBinaryStream.SaveToFile strMP3Filename, adSaveCreateOverWrite
        stmBinaryStream.Close
        Set stmBinaryStream = Nothing
        strID3v2Version = "none"
        lngFileSize = (lngFileSize - lngID3v2Size) - intTagIDSize
    End If
    
    Set ts = Nothing
    Set fso = Nothing
End Sub

'WriteID3v2 - Writes ID3v2 version 3.0 tag to the file.
Public Sub WriteID3v2()
    Dim stmBinaryStream As New Stream
    Dim strID3v2Tag As String, strFramesTag As String
    Dim lngTXXXCount As Long, lngWCOMCount As Long, lngWOARCount As Long, lngWXXXCount As Long
    Dim strTXXXSplit() As String, strWCOMSplit() As String, strWOARSplit() As String, strWXXXSplit() As String
    Dim bytID3v2Tag() As Byte, bytReadBinaryFile() As Byte
    
    StripID3v2
    strID3v2Tag = ""
    strFramesTag = ""
    If Len(strUFIDOwner) > 0 And Len(strUFIDIdentifier) > 0 Then
        strFramesTag = strFramesTag & "UFID" & GetBinaryInt(Len(strUFIDOwner) + Len(strUFIDIdentifier) + 1) & Chr(0) & Chr(0) & strUFIDOwner & Chr(0) & strUFIDIdentifier
    End If
    If Len(strAlbum) > 0 Then
        strFramesTag = strFramesTag & "TALB" & GetBinaryInt(Len(strAlbum) + 1) & Chr(0) & Chr(0) & Chr(0) & strAlbum
    End If
    If Len(strTBPM) > 0 Then
        strFramesTag = strFramesTag & "TBPM" & GetBinaryInt(Len(strTBPM) + 1) & Chr(0) & Chr(0) & Chr(0) & strTBPM
    End If
    If Len(strTCOM) > 0 Then
        strFramesTag = strFramesTag & "TCOM" & GetBinaryInt(Len(strTCOM) + 1) & Chr(0) & Chr(0) & Chr(0) & strTCOM
    End If
    If Len(lngGenreID) > 0 Then
        strFramesTag = strFramesTag & "TCON" & GetBinaryInt(Len(CStr(lngGenreID)) + 3) & Chr(0) & Chr(0) & Chr(0) & "(" & lngGenreID & ")"
    End If
    If Len(strTCOP) > 0 Then
        strFramesTag = strFramesTag & "TCOP" & GetBinaryInt(Len(strTCOP) + 1) & Chr(0) & Chr(0) & Chr(0) & strTCOP
    End If
    If Len(strTDAT) > 0 Then
        strFramesTag = strFramesTag & "TDAT" & GetBinaryInt(Len(strTDAT) + 1) & Chr(0) & Chr(0) & Chr(0) & strTDAT
    End If
    If Len(strTDLY) > 0 Then
        strFramesTag = strFramesTag & "TDLY" & GetBinaryInt(Len(strTDLY) + 1) & Chr(0) & Chr(0) & Chr(0) & strTDLY
    End If
    If Len(strTENC) > 0 Then
        strFramesTag = strFramesTag & "TENC" & GetBinaryInt(Len(strTENC) + 1) & Chr(0) & Chr(0) & Chr(0) & strTENC
    End If
    If Len(strTEXT) > 0 Then
        strFramesTag = strFramesTag & "TEXT" & GetBinaryInt(Len(strTEXT) + 1) & Chr(0) & Chr(0) & Chr(0) & strTEXT
    End If
    If Len(strTFLT) > 0 Then
        strFramesTag = strFramesTag & "TFLT" & GetBinaryInt(Len(strTFLT) + 1) & Chr(0) & Chr(0) & Chr(0) & strTFLT
    End If
    If Len(strTIME) > 0 Then
        strFramesTag = strFramesTag & "TIME" & GetBinaryInt(Len(strTIME) + 1) & Chr(0) & Chr(0) & Chr(0) & strTIME
    End If
    If Len(strTIT1) > 0 Then
        strFramesTag = strFramesTag & "TIT1" & GetBinaryInt(Len(strTIT1) + 1) & Chr(0) & Chr(0) & Chr(0) & strTIT1
    End If
    If Len(strTitle) > 0 Then
        strFramesTag = strFramesTag & "TIT2" & GetBinaryInt(Len(strTitle) + 1) & Chr(0) & Chr(0) & Chr(0) & strTitle
    End If
    If Len(strTIT3) > 0 Then
        strFramesTag = strFramesTag & "TIT3" & GetBinaryInt(Len(strTIT3) + 1) & Chr(0) & Chr(0) & Chr(0) & strTIT3
    End If
    If Len(strTKEY) > 0 Then
        strFramesTag = strFramesTag & "TKEY" & GetBinaryInt(Len(strTKEY) + 1) & Chr(0) & Chr(0) & Chr(0) & strTKEY
    End If
    If Len(strTLAN) > 0 Then
        strFramesTag = strFramesTag & "TLAN" & GetBinaryInt(Len(strTLAN) + 1) & Chr(0) & Chr(0) & Chr(0) & strTLAN
    End If
    If Len(strTLEN) > 0 Then
        strFramesTag = strFramesTag & "TLEN" & GetBinaryInt(Len(strTLEN) + 1) & Chr(0) & Chr(0) & Chr(0) & strTLEN
    End If
    If Len(strTMED) > 0 Then
        strFramesTag = strFramesTag & "TMED" & GetBinaryInt(Len(strTMED) + 1) & Chr(0) & Chr(0) & Chr(0) & strTMED
    End If
    If Len(strTOAL) > 0 Then
        strFramesTag = strFramesTag & "TOAL" & GetBinaryInt(Len(strTOAL) + 1) & Chr(0) & Chr(0) & Chr(0) & strTOAL
    End If
    If Len(strTOFN) > 0 Then
        strFramesTag = strFramesTag & "TOFN" & GetBinaryInt(Len(strTOFN) + 1) & Chr(0) & Chr(0) & Chr(0) & strTOFN
    End If
    If Len(strTOLY) > 0 Then
        strFramesTag = strFramesTag & "TOLY" & GetBinaryInt(Len(strTOLY) + 1) & Chr(0) & Chr(0) & Chr(0) & strTOLY
    End If
    If Len(strTOPE) > 0 Then
        strFramesTag = strFramesTag & "TOPE" & GetBinaryInt(Len(strTOPE) + 1) & Chr(0) & Chr(0) & Chr(0) & strTOPE
    End If
    If Len(strTORY) > 0 Then
        strFramesTag = strFramesTag & "TORY" & GetBinaryInt(Len(strTORY) + 1) & Chr(0) & Chr(0) & Chr(0) & strTORY
    End If
    If Len(strTOWN) > 0 Then
        strFramesTag = strFramesTag & "TOWN" & GetBinaryInt(Len(strTOWN) + 1) & Chr(0) & Chr(0) & Chr(0) & strTOWN
    End If
    If Len(strArtist) > 0 Then
        strFramesTag = strFramesTag & "TPE1" & GetBinaryInt(Len(strArtist) + 1) & Chr(0) & Chr(0) & Chr(0) & strArtist
    End If
    If Len(strTPE2) > 0 Then
        strFramesTag = strFramesTag & "TPE2" & GetBinaryInt(Len(strTPE2) + 1) & Chr(0) & Chr(0) & Chr(0) & strTPE2
    End If
    If Len(strTPE3) > 0 Then
        strFramesTag = strFramesTag & "TPE3" & GetBinaryInt(Len(strTPE3) + 1) & Chr(0) & Chr(0) & Chr(0) & strTPE3
    End If
    If Len(strTPE4) > 0 Then
        strFramesTag = strFramesTag & "TPE4" & GetBinaryInt(Len(strTPE4) + 1) & Chr(0) & Chr(0) & Chr(0) & strTPE4
    End If
    If Len(strTPOS) > 0 Then
        strFramesTag = strFramesTag & "TPOS" & GetBinaryInt(Len(strTPOS) + 1) & Chr(0) & Chr(0) & Chr(0) & strTPOS
    End If
    If Len(strTPUB) > 0 Then
        strFramesTag = strFramesTag & "TPUB" & GetBinaryInt(Len(strTPUB) + 1) & Chr(0) & Chr(0) & Chr(0) & strTPUB
    End If
    If Len(strTRCK) > 0 Then
        strFramesTag = strFramesTag & "TRCK" & GetBinaryInt(Len(strTRCK) + 1) & Chr(0) & Chr(0) & Chr(0) & strTRCK
    End If
    If Len(strTRDA) > 0 Then
        strFramesTag = strFramesTag & "TRDA" & GetBinaryInt(Len(strTRDA) + 1) & Chr(0) & Chr(0) & Chr(0) & strTRDA
    End If
    If Len(strTRSN) > 0 Then
        strFramesTag = strFramesTag & "TRSN" & GetBinaryInt(Len(strTRSN) + 1) & Chr(0) & Chr(0) & Chr(0) & strTRSN
    End If
    If Len(strTRSO) > 0 Then
        strFramesTag = strFramesTag & "TRSO" & GetBinaryInt(Len(strTRSO) + 1) & Chr(0) & Chr(0) & Chr(0) & strTRSO
    End If
    If Len(strTSIZ) > 0 Then
        strFramesTag = strFramesTag & "TSIZ" & GetBinaryInt(Len(strTSIZ) + 1) & Chr(0) & Chr(0) & Chr(0) & strTSIZ
    End If
    If Len(strTSRC) > 0 Then
        strFramesTag = strFramesTag & "TSRC" & GetBinaryInt(Len(strTSRC) + 1) & Chr(0) & Chr(0) & Chr(0) & strTSRC
    End If
    If Len(strTSSE) > 0 Then
        strFramesTag = strFramesTag & "TSSE" & GetBinaryInt(Len(strTSSE) + 1) & Chr(0) & Chr(0) & Chr(0) & strTSSE
    End If
    If Len(strTYER) > 0 Then
        strFramesTag = strFramesTag & "TYER" & GetBinaryInt(Len(strTYER) + 1) & Chr(0) & Chr(0) & Chr(0) & strTYER
    End If
    If Len(strTXXX) > 0 Then
        If Mid(strTXXX, Len(strTXXX), 1) = Chr(0) Then strTXXX = Mid(strTXXX, 1, Len(strTXXX) - 1)
        strTXXXSplit = Split(strTXXX, Chr(0))
        For lngTXXXCount = 0 To UBound(strTXXXSplit, 1) Step 2
            If lngTXXXCount = UBound(strTXXXSplit, 1) Then Exit For
            strFramesTag = strFramesTag & "TXXX" & GetBinaryInt(Len(strTXXXSplit(lngTXXXCount)) + Len(strTXXXSplit(lngTXXXCount + 1)) + 2) & Chr(0) & Chr(0) & Chr(0) & strTXXXSplit(lngTXXXCount) & Chr(0) & strTXXXSplit(lngTXXXCount + 1)
        Next
    End If
    If Len(strWCOM) > 0 Then
        If Mid(strWCOM, Len(strWCOM), 1) = Chr(0) Then strWCOM = Mid(strWCOM, 1, Len(strWCOM) - 1)
        strWCOMSplit = Split(strWCOM, Chr(0))
        For lngWCOMCount = 0 To UBound(strWCOMSplit, 1)
            strFramesTag = strFramesTag & "WCOM" & GetBinaryInt(Len(strWCOMSplit(lngWCOMCount))) & Chr(0) & Chr(0) & strWCOMSplit(lngWCOMCount)
        Next
    End If
    If Len(strWCOP) > 0 Then
        strFramesTag = strFramesTag & "WCOP" & GetBinaryInt(Len(strWCOP)) & Chr(0) & Chr(0) & Chr(0) & strWCOP
    End If
    If Len(strWOAF) > 0 Then
        strFramesTag = strFramesTag & "WOAF" & GetBinaryInt(Len(strWOAF)) & Chr(0) & Chr(0) & Chr(0) & strWOAF
    End If
    If Len(strWOAR) > 0 Then
        If Mid(strWOAR, Len(strWOAR), 1) = Chr(0) Then strWOAR = Mid(strWOAR, 1, Len(strWOAR) - 1)
        strWOARSplit = Split(strWOAR, Chr(0))
        For lngWOARCount = 0 To UBound(strWOARSplit, 1)
            strFramesTag = strFramesTag & "WOAR" & GetBinaryInt(Len(strWOARSplit(lngWOARCount))) & Chr(0) & Chr(0) & strWOARSplit(lngWOARCount)
        Next
    End If
    If Len(strWOAS) > 0 Then
        strFramesTag = strFramesTag & "WOAS" & GetBinaryInt(Len(strWOAS)) & Chr(0) & Chr(0) & Chr(0) & strWOAS
    End If
    If Len(strWORS) > 0 Then
        strFramesTag = strFramesTag & "WORS" & GetBinaryInt(Len(strWORS)) & Chr(0) & Chr(0) & Chr(0) & strWORS
    End If
    If Len(strWPAY) > 0 Then
        strFramesTag = strFramesTag & "WPAY" & GetBinaryInt(Len(strWPAY)) & Chr(0) & Chr(0) & Chr(0) & strWPAY
    End If
    If Len(strWPUB) > 0 Then
        strFramesTag = strFramesTag & "WPUB" & GetBinaryInt(Len(strWPUB)) & Chr(0) & Chr(0) & Chr(0) & strWPUB
    End If
    If Len(strWXXX) > 0 Then
        If Mid(strWXXX, Len(strWXXX), 1) = Chr(0) Then strWXXX = Mid(strWXXX, 1, Len(strWXXX) - 1)
        strWXXXSplit = Split(strWXXX, Chr(0))
        For lngWXXXCount = 0 To UBound(strWXXXSplit, 1) Step 2
            If lngWXXXCount = UBound(strWXXXSplit, 1) Then Exit For
            strFramesTag = strFramesTag & "WXXX" & GetBinaryInt(Len(strWXXXSplit(lngWXXXCount)) + Len(strWXXXSplit(lngWXXXCount + 1)) + 2) & Chr(0) & Chr(0) & Chr(0) & strWXXXSplit(lngWXXXCount) & Chr(0) & strWXXXSplit(lngWXXXCount + 1)
        Next
    End If
    If Len(strComment) > 0 Then
        strFramesTag = strFramesTag & "COMM" & GetBinaryInt(Len(strComment) + 5) & Chr(0) & Chr(0) & Chr(0) & "ENG" & Chr(0) & strComment
    End If
    strID3v2Tag = "ID3" & Chr(3) & Chr(0) & Chr(0) & GetUnsynchedBinaryInt(Len(strFramesTag)) & strFramesTag
    bytID3v2Tag = Stream_StringToBinary(strID3v2Tag)
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    stmBinaryStream.LoadFromFile strMP3Filename
    bytReadBinaryFile = stmBinaryStream.Read(lngFileSize)
    stmBinaryStream.Close
    Set stmBinaryStream = Nothing
    Set stmBinaryStream = New Stream
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    stmBinaryStream.Write bytID3v2Tag
    stmBinaryStream.Write bytReadBinaryFile
    stmBinaryStream.SaveToFile strMP3Filename, adSaveCreateOverWrite
    stmBinaryStream.Close
    
    Set stmBinaryStream = Nothing
End Sub

'ToBin - Converts a string to a string of its binary representation.
Private Function ToBin(strIn As String) As String
    Dim lngLen As Long, lngCount As Long
    Dim intVal As Integer, intBitCount As Integer
    
    If Len(strIn) = 0 Then
        ToBin = ""
        Exit Function
    End If
    lngLen = Len(strIn) * 8
    ToBin = ""
    For lngCount = Len(strIn) To 1 Step -1
        intVal = Asc(Mid(strIn, lngCount, 1))
        For intBitCount = 1 To 8
            ToBin = CStr(intVal Mod 2) & ToBin
            intVal = intVal \ 2
        Next
    Next
End Function

'ReadHeader - Reads the header, and returns the bitrate from it.
Private Function ReadHeader(strReadBin As String) As Integer
    Dim intBitrate As Integer
    Dim strEmphasis As String
    
    Select Case Mid(strReadBin, 12, 2)
        Case "00"
            strMPEGVersion = "2.5"
        Case "01"
            strMPEGVersion = "Reserved"
            ReadHeader = -1
            blnIsMP3 = False
            Exit Function
        Case "10"
            strMPEGVersion = "2"
        Case "11"
            strMPEGVersion = "1"
    End Select

    Select Case Mid(strReadBin, 14, 2)
        Case "00"
            strLayer = "Reserved"
            ReadHeader = -1
            blnIsMP3 = False
            Exit Function
        Case "01"
            strLayer = "III"
        Case "10"
            strLayer = "II"
        Case "11"
            strLayer = "I"
    End Select
    
    If Mid(strReadBin, 16, 1) = "0" Then blnProtected = True Else blnProtected = False

    intBitrate = GetBitrate(Mid(strReadBin, 17, 4), strMPEGVersion, strLayer)
    If intBitrate = -1 Then
        ReadHeader = -1
        blnIsMP3 = False
        Exit Function
    Else
        ReadHeader = intBitrate
    End If

    Select Case Mid(strReadBin, 21, 2)
        Case "00"
            Select Case strMPEGVersion
                Case "1"
                    strSampleRate = "44100"
                Case "2"
                    strSampleRate = "22050"
                Case "2.5"
                    strSampleRate = "11025"
            End Select
        Case "01"
            Select Case strMPEGVersion
                Case "1"
                    strSampleRate = "48000"
                Case "2"
                    strSampleRate = "24000"
                Case "2.5"
                    strSampleRate = "16000"
            End Select
        Case "10"
            Select Case strMPEGVersion
                Case "1"
                    strSampleRate = "32000"
                Case "2"
                    strSampleRate = "16000"
                Case "2.5"
                    strSampleRate = "8000"
            End Select
        Case "11"
            strSampleRate = "Reserved"
            ReadHeader = -1
            blnIsMP3 = False
            Exit Function
    End Select

    If Mid(strReadBin, 23, 1) = "0" Then blnPadding = False Else blnPadding = True

    If Mid(strReadBin, 24, 1) = "0" Then blnPrivateBit = False Else blnPrivateBit = True

    Select Case Mid(strReadBin, 25, 2)
        Case "00"
            strChannelMode = "Stereo"
        Case "01"
            strChannelMode = "Joint Stereo"
        Case "10"
            strChannelMode = "Dual Channel"
        Case "11"
            strChannelMode = "Single Channel"
    End Select

    Select Case Mid(strReadBin, 27, 2)
        Case "00"
            Select Case strLayer
                Case "III"
                    strIntensityStereoBands = "n/a"
                    strIntensityStereo = "Off"
                    strMSStereo = "Off"
                Case Else
                    strIntensityStereoBands = "Bands 4 to 31"
                    strIntensityStereo = "n/a"
                    strMSStereo = "n/a"
            End Select
        Case "01"
            Select Case strLayer
                Case "III"
                    strIntensityStereoBands = "n/a"
                    strIntensityStereo = "On"
                    strMSStereo = "Off"
                Case Else
                    strIntensityStereoBands = "Bands 8 to 31"
                    strIntensityStereo = "n/a"
                    strMSStereo = "n/a"
            End Select
        Case "10"
            Select Case strLayer
                Case "III"
                    strIntensityStereoBands = "n/a"
                    strIntensityStereo = "Off"
                    strMSStereo = "On"
                Case Else
                    strIntensityStereoBands = "Bands 12 to 31"
                    strIntensityStereo = "n/a"
                    strMSStereo = "n/a"
            End Select
        Case "11"
            Select Case strLayer
                Case "III"
                    strIntensityStereoBands = "n/a"
                    strIntensityStereo = "On"
                    strMSStereo = "On"
                Case Else
                    strIntensityStereoBands = "Bands 16 to 31"
                    strIntensityStereo = "n/a"
                    strMSStereo = "n/a"
            End Select
    End Select

    If Mid(strReadBin, 29, 1) = "0" Then blnCopyright = False Else blnCopyright = True

    If Mid(strReadBin, 30, 1) = "0" Then blnOriginal = False Else blnOriginal = True

    Select Case Mid(strReadBin, 31, 2)
        Case "00"
            strEmphasis = "None"
        Case "01"
            strEmphasis = "50/15 ms"
        Case "10"
            strEmphasis = "Reserved"
            ReadHeader = -1
            blnIsMP3 = False
            Exit Function
        Case "11"
            strEmphasis = "CCIT J.17"
    End Select
End Function

'GetBitrate - Determines the bitrate, based on the MPEG Version, Layer, and string binary representation of a bitrate.
Private Function GetBitrate(strBin As String, strMPEGVersion As String, strLayer As String) As Integer
    Select Case strBin
        Case "0000"
            GetBitrate = 0
        Case "0001"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 32
                        Case "II"
                            GetBitrate = 32
                        Case "III"
                            GetBitrate = 32
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 32
                        Case Else
                            GetBitrate = 8
                    End Select
            End Select
        Case "0010"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 64
                        Case "II"
                            GetBitrate = 48
                        Case "III"
                            GetBitrate = 40
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 48
                        Case Else
                            GetBitrate = 16
                    End Select
            End Select
        Case "0011"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 96
                        Case "II"
                            GetBitrate = 56
                        Case "III"
                            GetBitrate = 48
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 56
                        Case Else
                            GetBitrate = 24
                    End Select
            End Select
        Case "0100"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 128
                        Case "II"
                            GetBitrate = 64
                        Case "III"
                            GetBitrate = 56
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 64
                        Case Else
                            GetBitrate = 32
                    End Select
            End Select
        Case "0101"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 160
                        Case "II"
                            GetBitrate = 80
                        Case "III"
                            GetBitrate = 64
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 80
                        Case Else
                            GetBitrate = 40
                    End Select
            End Select
        Case "0110"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 192
                        Case "II"
                            GetBitrate = 96
                        Case "III"
                            GetBitrate = 80
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 96
                        Case Else
                            GetBitrate = 48
                    End Select
            End Select
        Case "0111"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 224
                        Case "II"
                            GetBitrate = 112
                        Case "III"
                            GetBitrate = 96
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 112
                        Case Else
                            GetBitrate = 56
                    End Select
            End Select
        Case "1000"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 256
                        Case "II"
                            GetBitrate = 128
                        Case "III"
                            GetBitrate = 112
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 128
                        Case Else
                            GetBitrate = 64
                    End Select
            End Select
        Case "1001"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 288
                        Case "II"
                            GetBitrate = 160
                        Case "III"
                            GetBitrate = 128
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 144
                        Case Else
                            GetBitrate = 80
                    End Select
            End Select
        Case "1010"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 320
                        Case "II"
                            GetBitrate = 192
                        Case "III"
                            GetBitrate = 160
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 160
                        Case Else
                            GetBitrate = 96
                    End Select
            End Select
        Case "1011"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 352
                        Case "II"
                            GetBitrate = 224
                        Case "III"
                            GetBitrate = 192
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 176
                        Case Else
                            GetBitrate = 112
                    End Select
            End Select
        Case "1100"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 384
                        Case "II"
                            GetBitrate = 256
                        Case "III"
                            GetBitrate = 224
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 192
                        Case Else
                            GetBitrate = 128
                    End Select
            End Select
        Case "1101"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 416
                        Case "II"
                            GetBitrate = 320
                        Case "III"
                            GetBitrate = 256
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 224
                        Case Else
                            GetBitrate = 144
                    End Select
            End Select
        Case "1110"
            Select Case strMPEGVersion
                Case "1"
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 448
                        Case "II"
                            GetBitrate = 384
                        Case "III"
                            GetBitrate = 320
                    End Select
                Case Else
                    Select Case strLayer
                        Case "I"
                            GetBitrate = 256
                        Case Else
                            GetBitrate = 160
                    End Select
            End Select
        Case "1111"
            GetBitrate = -1
    End Select
End Function

'NullTruncate - Truncates a string with a null character in it.
Private Function NullTruncate(strIn As String) As String
    Dim lngNullTerm As Long
    
    lngNullTerm = InStr(strIn, Chr(0))
    If lngNullTerm = 0 Then
        NullTruncate = strIn
    Else
        NullTruncate = Mid(strIn, 1, lngNullTerm - 1)
    End If
End Function

'GetBinaryInt - Converts an integer to 4 byte integer.
'GetBinaryInt - Converts an integer to 4 byte integer.
Private Function GetBinaryInt(lngNumber As Long) As String
    Dim dblNumber As Double
    
    dblNumber = lngNumber
    If dblNumber < 0 Then
        dblNumber = dblNumber + 2147483648#
        GetBinaryInt = Chr(Int(dblNumber / 16777216) + 128)
    Else
        GetBinaryInt = Chr(Int(dblNumber / 16777216))
    End If
    dblNumber = dblNumber Mod 16777216
    GetBinaryInt = Chr(Int(dblNumber / 65536)) & GetBinaryInt
    dblNumber = dblNumber Mod 65536
    GetBinaryInt = Chr(Int(dblNumber / 256)) & GetBinaryInt
    dblNumber = dblNumber Mod 256
    GetBinaryInt = Chr(dblNumber) & GetBinaryInt
End Function

'GetUnsynchedBinaryInt - Converts an integer to 4 byte synchsafe integer.
Private Function GetUnsynchedBinaryInt(lngNumber As Long) As String
    GetUnsynchedBinaryInt = Chr(Int(lngNumber / 2097152))
    lngNumber = lngNumber Mod 2097152
    GetUnsynchedBinaryInt = GetUnsynchedBinaryInt & Chr(Int(lngNumber / 16384))
    lngNumber = lngNumber Mod 16384
    GetUnsynchedBinaryInt = GetUnsynchedBinaryInt & Chr(Int(lngNumber / 128))
    lngNumber = lngNumber Mod 128
    GetUnsynchedBinaryInt = GetUnsynchedBinaryInt & Chr(lngNumber)
End Function

'Stream_StringToBinary Function
'2003 Antonin Foller, http://www.pstruh.cz
'Text - string parameter To convert To binary data
Private Function Stream_StringToBinary(strIn As String) As Byte()
    'Create Stream object
    Dim stmBinaryStream As New Stream

    'Specify stream type - we want To save text/string data.
    stmBinaryStream.Type = adTypeText

    'Specify charset For the source text (unicode) data.
    stmBinaryStream.Charset = "windows-1252"

    'Open the stream And write text/string data To the object
    stmBinaryStream.Open
    stmBinaryStream.WriteText strIn

    'Change stream type To binary
    stmBinaryStream.Position = 0
    stmBinaryStream.Type = adTypeBinary

    'Ignore first two bytes - sign of
    stmBinaryStream.Position = 0

    'Open the stream And get binary data from the object
    Stream_StringToBinary = stmBinaryStream.Read
    
    Set stmBinaryStream = Nothing
End Function

Public Function GetGenre(intGenreID As Integer) As String
    Select Case intGenreID
        Case 0
            GetGenre = "Blues"
        Case 1
            GetGenre = "Classic Rock"
        Case 2
            GetGenre = "Country"
        Case 3
            GetGenre = "Dance"
        Case 4
            GetGenre = "Disco"
        Case 5
            GetGenre = "Funk"
        Case 6
            GetGenre = "Grunge"
        Case 7
            GetGenre = "Hip-Hop"
        Case 8
            GetGenre = "Jazz"
        Case 9
            GetGenre = "Metal"
        Case 10
            GetGenre = "New Age"
        Case 11
            GetGenre = "Oldies"
        Case 12
            GetGenre = "Other"
        Case 13
            GetGenre = "Pop"
        Case 14
            GetGenre = "R&B"
        Case 15
            GetGenre = "Rap"
        Case 16
            GetGenre = "Reggae"
        Case 17
            GetGenre = "Rock"
        Case 18
            GetGenre = "Techno"
        Case 19
            GetGenre = "Industrial"
        Case 20
            GetGenre = "Alternative"
        Case 21
            GetGenre = "Ska"
        Case 22
            GetGenre = "Death Metal"
        Case 23
            GetGenre = "Pranks"
        Case 24
            GetGenre = "Soundtrack"
        Case 25
            GetGenre = "Euro-Techno"
        Case 26
            GetGenre = "Ambient"
        Case 27
            GetGenre = "Trip-Hop"
        Case 28
            GetGenre = "Vocal"
        Case 29
            GetGenre = "Jazz+Funk"
        Case 30
            GetGenre = "Fusion"
        Case 31
            GetGenre = "Trance"
        Case 32
            GetGenre = "Classical"
        Case 33
            GetGenre = "Instrumental"
        Case 34
            GetGenre = "Acid"
        Case 35
            GetGenre = "House"
        Case 36
            GetGenre = "Game"
        Case 37
            GetGenre = "Sound Clip"
        Case 38
            GetGenre = "Gospel"
        Case 39
            GetGenre = "Noise"
        Case 40
            GetGenre = "Alternative Rock"
        Case 41
            GetGenre = "Bass"
        Case 42
            GetGenre = "Soul"
        Case 43
            GetGenre = "Punk"
        Case 44
            GetGenre = "Space"
        Case 45
            GetGenre = "Meditative"
        Case 46
            GetGenre = "Instrumental Pop"
        Case 47
            GetGenre = "Instrumental Rock"
        Case 48
            GetGenre = "Ethnic"
        Case 49
            GetGenre = "Gothic"
        Case 50
            GetGenre = "Darkwave"
        Case 51
            GetGenre = "Techno-Industrial"
        Case 52
            GetGenre = "Electric"
        Case 53
            GetGenre = "Pop-Folk"
        Case 54
            GetGenre = "Eurodance"
        Case 55
            GetGenre = "Dream"
        Case 56
            GetGenre = "Southern Rock"
        Case 57
            GetGenre = "Comedy"
        Case 58
            GetGenre = "Cult"
        Case 59
            GetGenre = "Gangsta"
        Case 60
            GetGenre = "Top 40"
        Case 61
            GetGenre = "Christian Rap"
        Case 62
            GetGenre = "Pop/Funk"
        Case 63
            GetGenre = "Jungle"
        Case 64
            GetGenre = "Native US"
        Case 65
            GetGenre = "Cabaret"
        Case 66
            GetGenre = "New Wave"
        Case 67
            GetGenre = "Psychadelic"
        Case 68
            GetGenre = "Rave"
        Case 69
            GetGenre = "Showtunes"
        Case 70
            GetGenre = "Trailer"
        Case 71
            GetGenre = "Lo-Fi"
        Case 72
            GetGenre = "Tribal"
        Case 73
            GetGenre = "Acid Punk"
        Case 74
            GetGenre = "Acid Jazz"
        Case 75
            GetGenre = "Polka"
        Case 76
            GetGenre = "Retro"
        Case 77
            GetGenre = "Musical"
        Case 78
            GetGenre = "Rock & Roll"
        Case 79
            GetGenre = "Hard Rock"
        Case 80
            GetGenre = "Folk"
        Case 81
            GetGenre = "Folk-Rock"
        Case 82
            GetGenre = "National Folk"
        Case 83
            GetGenre = "Swing"
        Case 84
            GetGenre = "Fast Fusion"
        Case 85
            GetGenre = "Bebob"
        Case 86
            GetGenre = "Latin"
        Case 87
            GetGenre = "Revival"
        Case 88
            GetGenre = "Celtic"
        Case 89
            GetGenre = "Bluegrass"
        Case 90
            GetGenre = "Avantgarde"
        Case 91
            GetGenre = "Gothic Rock"
        Case 92
            GetGenre = "Progressive Rock"
        Case 93
            GetGenre = "Psychedelic Rock"
        Case 94
            GetGenre = "Symphonic Rock"
        Case 95
            GetGenre = "Slow Rock"
        Case 96
            GetGenre = "Big Band"
        Case 97
            GetGenre = "Chorus"
        Case 98
            GetGenre = "Easy Listening"
        Case 99
            GetGenre = "Acoustic"
        Case 100
            GetGenre = "Humor"
        Case 101
            GetGenre = "Speech"
        Case 102
            GetGenre = "Chanson"
        Case 103
            GetGenre = "Opera"
        Case 104
            GetGenre = "Chamber Music"
        Case 105
            GetGenre = "Sonata"
        Case 106
            GetGenre = "Symphony"
        Case 107
            GetGenre = "Booty Bass"
        Case 108
            GetGenre = "Primus"
        Case 109
            GetGenre = "Porn Groove"
        Case 110
            GetGenre = "Satire"
        Case 111
            GetGenre = "Slow Jam"
        Case 112
            GetGenre = "Club"
        Case 113
            GetGenre = "Tango"
        Case 114
            GetGenre = "Samba"
        Case 115
            GetGenre = "Folklore"
        Case 116
            GetGenre = "Ballad"
        Case 117
            GetGenre = "Power Ballad"
        Case 118
            GetGenre = "Rhythmic Soul"
        Case 119
            GetGenre = "Freestyle"
        Case 120
            GetGenre = "Duet"
        Case 121
            GetGenre = "Punk Rock"
        Case 122
            GetGenre = "Drum Solo"
        Case 123
            GetGenre = "Acapella"
        Case 124
            GetGenre = "Euro-House"
        Case 125
            GetGenre = "Dance Hall"
        Case 126
            GetGenre = "Goa"
        Case 127
            GetGenre = "Drum & Bass"
        Case 128
            GetGenre = "Club-House"
        Case 129
            GetGenre = "Hardcore"
        Case 130
            GetGenre = "Terror"
        Case 131
            GetGenre = "Indie"
        Case 132
            GetGenre = "BritPop"
        Case 133
            GetGenre = "Negerpunk"
        Case 134
            GetGenre = "Polsk Punk"
        Case 135
            GetGenre = "Beat"
        Case 136
            GetGenre = "Christian Gangsta Rap"
        Case 137
            GetGenre = "Heavy Metal"
        Case 138
            GetGenre = "Black Metal"
        Case 139
            GetGenre = "Crossover"
        Case 140
            GetGenre = "Contemporary Christian"
        Case 141
            GetGenre = "Christian Rock"
        Case 142
            GetGenre = "Merengue"
        Case 143
            GetGenre = "Salsa"
        Case 144
            GetGenre = "Trash Metal"
        Case 145
            GetGenre = "Anime"
        Case 146
            GetGenre = "Jpop"
        Case 147
            GetGenre = "Synthpop"
        Case Else
            GetGenre = ""
    End Select
End Function

Public Function GetGenreID(strGenre As String) As Integer
    Select Case strGenre
        Case "Blues"
            GetGenreID = 0
        Case "Classic Rock"
            GetGenreID = 1
        Case "Country"
            GetGenreID = 2
        Case "Dance"
            GetGenreID = 3
        Case "Disco"
            GetGenreID = 4
        Case "Funk"
            GetGenreID = 5
        Case "Grunge"
            GetGenreID = 6
        Case "Hip-Hop"
            GetGenreID = 7
        Case "Jazz"
            GetGenreID = 8
        Case "Metal"
            GetGenreID = 9
        Case "New Age"
            GetGenreID = 10
        Case "Oldies"
            GetGenreID = 11
        Case "Other"
            GetGenreID = 12
        Case "Pop"
            GetGenreID = 13
        Case "R&B"
            GetGenreID = 14
        Case "Rap"
            GetGenreID = 15
        Case "Reggae"
            GetGenreID = 16
        Case "Rock"
            GetGenreID = 17
        Case "Techno"
            GetGenreID = 18
        Case "Industrial"
            GetGenreID = 19
        Case "Alternative"
            GetGenreID = 20
        Case "Ska"
            GetGenreID = 21
        Case "Death Metal"
            GetGenreID = 22
        Case "Pranks"
            GetGenreID = 23
        Case "Soundtrack"
            GetGenreID = 24
        Case "Euro-Techno"
            GetGenreID = 25
        Case "Ambient"
            GetGenreID = 26
        Case "Trip-Hop"
            GetGenreID = 27
        Case "Vocal"
            GetGenreID = 28
        Case "Jazz+Funk"
            GetGenreID = 29
        Case "Fusion"
            GetGenreID = 30
        Case "Trance"
            GetGenreID = 31
        Case "Classical"
            GetGenreID = 32
        Case "Instrumental"
            GetGenreID = 33
        Case "Acid"
            GetGenreID = 34
        Case "House"
            GetGenreID = 35
        Case "Game"
            GetGenreID = 36
        Case "Sound Clip"
            GetGenreID = 37
        Case "Gospel"
            GetGenreID = 38
        Case "Noise"
            GetGenreID = 39
        Case "Alternative Rock"
            GetGenreID = 40
        Case "Bass"
            GetGenreID = 41
        Case "Soul"
            GetGenreID = 42
        Case "Punk"
            GetGenreID = 43
        Case "Space"
            GetGenreID = 44
        Case "Meditative"
            GetGenreID = 45
        Case "Instrumental Pop"
            GetGenreID = 46
        Case "Instrumental Rock"
            GetGenreID = 47
        Case "Ethnic"
            GetGenreID = 48
        Case "Gothic"
            GetGenreID = 49
        Case "Darkwave"
            GetGenreID = 50
        Case "Techno-Industrial"
            GetGenreID = 51
        Case "Electric"
            GetGenreID = 52
        Case "Pop-Folk"
            GetGenreID = 53
        Case "Eurodance"
            GetGenreID = 54
        Case "Dream"
            GetGenreID = 55
        Case "Southern Rock"
            GetGenreID = 56
        Case "Comedy"
            GetGenreID = 57
        Case "Cult"
            GetGenreID = 58
        Case "Gangsta"
            GetGenreID = 59
        Case "Top 40"
            GetGenreID = 60
        Case "Christian Rap"
            GetGenreID = 61
        Case "Pop/Funk"
            GetGenreID = 62
        Case "Jungle"
            GetGenreID = 63
        Case "Native US"
            GetGenreID = 64
        Case "Cabaret"
            GetGenreID = 65
        Case "New Wave"
            GetGenreID = 66
        Case "Psychadelic"
            GetGenreID = 67
        Case "Rave"
            GetGenreID = 68
        Case "Showtunes"
            GetGenreID = 69
        Case "Trailer"
            GetGenreID = 70
        Case "Lo-Fi"
            GetGenreID = 71
        Case "Tribal"
            GetGenreID = 72
        Case "Acid Punk"
            GetGenreID = 73
        Case "Acid Jazz"
            GetGenreID = 74
        Case "Polka"
            GetGenreID = 75
        Case "Retro"
            GetGenreID = 76
        Case "Musical"
            GetGenreID = 77
        Case "Rock & Roll"
            GetGenreID = 78
        Case "Hard Rock"
            GetGenreID = 79
        Case "Folk"
            GetGenreID = 80
        Case "Folk-Rock"
            GetGenreID = 81
        Case "National Folk"
            GetGenreID = 82
        Case "Swing"
            GetGenreID = 83
        Case "Fast Fusion"
            GetGenreID = 84
        Case "Bebob"
            GetGenreID = 85
        Case "Latin"
            GetGenreID = 86
        Case "Revival"
            GetGenreID = 87
        Case "Celtic"
            GetGenreID = 88
        Case "Bluegrass"
            GetGenreID = 89
        Case "Avantgarde"
            GetGenreID = 90
        Case "Gothic Rock"
            GetGenreID = 91
        Case "Progressive Rock"
            GetGenreID = 92
        Case "Psychedelic Rock"
            GetGenreID = 93
        Case "Symphonic Rock"
            GetGenreID = 94
        Case "Slow Rock"
            GetGenreID = 95
        Case "Big Band"
            GetGenreID = 96
        Case "Chorus"
            GetGenreID = 97
        Case "Easy Listening"
            GetGenreID = 98
        Case "Acoustic"
            GetGenreID = 99
        Case "Humor"
            GetGenreID = 100
        Case "Speech"
            GetGenreID = 101
        Case "Chanson"
            GetGenreID = 102
        Case "Opera"
            GetGenreID = 103
        Case "Chamber Music"
            GetGenreID = 104
        Case "Sonata"
            GetGenreID = 105
        Case "Symphony"
            GetGenreID = 106
        Case "Booty Bass"
            GetGenreID = 107
        Case "Primus"
            GetGenreID = 108
        Case "Porn Groove"
            GetGenreID = 109
        Case "Satire"
            GetGenreID = 110
        Case "Slow Jam"
            GetGenreID = 111
        Case "Club"
            GetGenreID = 112
        Case "Tango"
            GetGenreID = 113
        Case "Samba"
            GetGenreID = 114
        Case "Folklore"
            GetGenreID = 115
        Case "Ballad"
            GetGenreID = 116
        Case "Power Ballad"
            GetGenreID = 117
        Case "Rhythmic Soul"
            GetGenreID = 118
        Case "Freestyle"
            GetGenreID = 119
        Case "Duet"
            GetGenreID = 120
        Case "Punk Rock"
            GetGenreID = 121
        Case "Drum Solo"
            GetGenreID = 122
        Case "Acapella"
            GetGenreID = 123
        Case "Euro-House"
            GetGenreID = 124
        Case "Dance Hall"
            GetGenreID = 125
        Case "Goa"
            GetGenreID = 126
        Case "Drum & Bass"
            GetGenreID = 127
        Case "Club-House"
            GetGenreID = 128
        Case "Hardcore"
            GetGenreID = 129
        Case "Terror"
            GetGenreID = 130
        Case "Indie"
            GetGenreID = 131
        Case "BritPop"
            GetGenreID = 132
        Case "Negerpunk"
            GetGenreID = 133
        Case "Polsk Punk"
            GetGenreID = 134
        Case "Beat"
            GetGenreID = 135
        Case "Christian Gangsta Rap"
            GetGenreID = 136
        Case "Heavy Metal"
            GetGenreID = 137
        Case "Black Metal"
            GetGenreID = 138
        Case "Crossover"
            GetGenreID = 139
        Case "Contemporary Christian"
            GetGenreID = 140
        Case "Christian Rock"
            GetGenreID = 141
        Case "Merengue"
            GetGenreID = 142
        Case "Salsa"
            GetGenreID = 143
        Case "Trash Metal"
            GetGenreID = 144
        Case "Anime"
            GetGenreID = 145
        Case "Jpop"
            GetGenreID = 146
        Case "Synthpop"
            GetGenreID = 147
        Case Else
            GetGenreID = 0
    End Select
End Function

'ProcessID3v2_2Tag - Processes an ID3v2.2 tag
Private Sub ProcessID3v2_2Tag(strID3v2Tag As String, strTagData As String)
    Dim lngNullTerm As Long, lngCloseParen As Long, lngInParenVal As Long, lngFoundSlash As Long, lngTIMHour As Long, lngTIMMinute As Long, lngAnotherNullTerm As Long
    Dim intCheckCount As Integer
    Dim strTP1Data As String, strTCOData As String, strTCOData2 As String, strInParen As String, strTPAData As String, strTRKData As String, strTRCData As String, strCountryCode As String, strRegistrantCode As String, strYearOfReference As String, strDesignationCode As String, strTYEData As String, strTDAData As String, strTDADay As String, strTDAMonth As String, strTIMData As String, strTBPData As String, strTCRData As String, strTLEData As String, strTSIData As String, strTDYData As String, strTKEData As String, strTORData As String, strTXXData As String, strWXXData As String, strLanguage As String, strCOMData As String
    Dim blnValidTRC As Boolean
    
    Select Case Mid(strID3v2Tag, 1, 3)
        Case "UFI" 'Unique File Identifier
            If Asc(Mid(strTagData, 1, 1)) > 0 Then
                lngNullTerm = InStr(strTagData, Chr(0))
                If lngNullTerm > 0 Then
                    strUFIDOwner = Mid(strTagData, 1, lngNullTerm - 1)
                    strUFIDIdentifier = NullTruncate(Mid(strTagData, lngNullTerm + 1, Len(strTagData) - lngNullTerm))
                End If
            End If
        Case "TT1" 'Content group description
            strTIT1 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TT2" 'Title/Songname/Content description
            strTIT2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTitle = strTIT2
        Case "TT3" 'Subtitle/Description refinement
            strTIT3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TP1" 'Lead artist(s)/Lead performer(s)/Soloist(s)/Performing group
            strTP1Data = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTPE1 = Split(strTP1Data, "/")
            strArtist = strTP1Data
        Case "TP2" 'Band/Orchestra/Accompaniment
            strTPE2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TP3" 'Conductor
            strTPE3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TP4" 'Interpreted, remixed, or otherwise modified by
            strTPE4 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TCM" 'Composer(s)
            strTCOM = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TXT" 'Lyricist(s)/text writer(s)
            strTEXT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TLA" 'Language(s)
            strTLAN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TCO" 'Content type
            strTCOData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTCOData2 = ""
            Do While Len(strTCOData) > 0
                If Mid(strTCOData, 1, 1) = "(" Then
                    If Mid(strTCOData, 2, 1) = "(" Then
                        strTCOData2 = strTCOData2 & Mid(strTCOData, 2, Len(strTCOData) - 1) & Chr(0)
                        strTCOData = ""
                    Else
                        lngCloseParen = InStr(strTCOData, ")")
                        If lngCloseParen > 2 Then
                            strInParen = Mid(strTCOData, 2, lngCloseParen - 2)
                            If strInParen = "RX" Or strInParen = "CR" Then
                                strTCOData2 = strTCOData2 & strInParen & Chr(0)
                                strTCOData = Mid(strTCOData, lngCloseParen + 1, Len(strTCOData) - lngCloseParen)
                            Else
                                If IsNumeric(strInParen) Then
                                    lngInParenVal = CLng(strInParen)
                                    If lngInParenVal >= 0 And lngInParenVal <= 147 Then
                                        strTCOData2 = strTCOData2 & CStr(lngInParenVal) & Chr(0)
                                        strTCOData = Mid(strTCOData, lngCloseParen + 1, Len(strTCOData) - lngCloseParen)
                                    Else
                                        strTCOData2 = strTCOData2 & strTCOData & Chr(0)
                                        strTCOData = ""
                                    End If
                                Else
                                    strTCOData2 = strTCOData2 & strTCOData & Chr(0)
                                    strTCOData = ""
                                End If
                            End If
                        Else
                            strTCOData2 = strTCOData2 & strTCOData & Chr(0)
                            strTCOData = ""
                        End If
                    End If
                Else
                    strTCOData2 = strTCOData2 & strTCOData & Chr(0)
                    strTCOData = ""
                End If
            Loop
            If Len(strTCOData2) > 0 Then
                strTCOData2 = Mid(strTCOData2, 2, Len(strTCOData2) - 1)
                strTCON = Split(strTCOData2, Chr(0))
                If IsNumeric(strTCON(0)) Then
                    lngGenreID = CLng(strTCON(0))
                End If
            End If
        Case "TAL" 'Album/Movie/Show title
            strTALB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strAlbum = strTALB
        Case "TPA" 'Part of a set
            strTPAData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTPAData) Then
                strTPOS = CStr(CLng(strTPAData))
            Else
                lngFoundSlash = InStr(strTPAData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTPAData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTPAData, lngFoundSlash + 1, Len(strTPAData) - lngFoundSlash)) Then
                        strTPOS = CStr(CLng(Mid(strTPAData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTPAData, lngFoundSlash + 1, Len(strTPAData) - lngFoundSlash)))
                    Else
                        strTPOS = ""
                    End If
                Else
                    strTPOS = ""
                End If
            End If
        Case "TRK" 'Track number/Position in set
            strTRKData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTRKData) Then
                strTRCK = CStr(CLng(strTRKData))
            Else
                lngFoundSlash = InStr(strTRKData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTRKData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTRKData, lngFoundSlash + 1, Len(strTRKData) - lngFoundSlash)) Then
                        strTRCK = CStr(CLng(Mid(strTRKData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTRKData, lngFoundSlash + 1, Len(strTRKData) - lngFoundSlash)))
                    Else
                        strTRCK = ""
                    End If
                Else
                    strTRCK = ""
                End If
            End If
        Case "TRC" 'ISRC
            strTRCData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTRCData)
                Case 12
                    strCountryCode = UCase(Mid(strTRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTRCData, 3, 3))
                    strYearOfReference = Mid(strTRCData, 6, 2)
                    strDesignationCode = Mid(strTRCData, 8, 5)
                Case 15
                    strCountryCode = UCase(Mid(strTRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTRCData, 4, 3))
                    strYearOfReference = Mid(strTRCData, 8, 2)
                    strDesignationCode = Mid(strTRCData, 11, 5)
                Case 16
                    strCountryCode = UCase(Mid(strTRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTRCData, 7, 3))
                    strYearOfReference = Mid(strTRCData, 10, 2)
                    strDesignationCode = Mid(strTRCData, 12, 5)
                Case 19
                    strCountryCode = UCase(Mid(strTRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTRCData, 8, 3))
                    strYearOfReference = Mid(strTRCData, 12, 2)
                    strDesignationCode = Mid(strTRCData, 15, 5)
                Case Else
                    strCountryCode = "AB"
                    strRegistrantCode = "ABC"
                    strYearOfReference = "AB"
                    strDesignationCode = "ABCDE"
            End Select
            If Not (IsNumeric(strYearOfReference) And IsNumeric(strDesignationCode)) Then
                strTSRC = ""
            Else
                blnValidTRC = True
                For intCheckCount = 1 To 3
                    If intCheckCount < 3 Then
                        If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strCountryCode, intCheckCount, 1)) = 0 Then
                            blnValidTRC = False
                            Exit For
                        End If
                    End If
                Next
                If blnValidTRC Then
                    strTSRC = strCountryCode & strRegistrantCode & strYearOfReference & strDesignationCode
                Else
                    strTSRC = ""
                End If
            End If
        Case "TYE" 'Year
            strTYEData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTYEData) Then
                strTYER = CStr(CLng(strTYEData))
            Else
                strTYER = ""
            End If
            strSongYear = strTYER
        Case "TDA" 'Date
            strTDAData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If Len(strTDAData) = 4 And IsNumeric(strTDAData) Then
                strTDADay = Mid(strTDAData, 1, 2)
                strTDAMonth = Mid(strTDAData, 3, 2)
                Select Case strTDAMonth
                    Case "01", "03", "05", "07", "08", "10", "12"
                        If CLng(strTDADay) >= 1 And CLng(strTDADay) <= 31 Then
                            strTDAT = strTDAData
                        Else
                            strTDAT = ""
                        End If
                    Case "04", "06", "09", "11"
                        If CLng(strTDADay) >= 1 And CLng(strTDADay) <= 30 Then
                            strTDAT = strTDAData
                        Else
                            strTDAT = ""
                        End If
                    Case "02"
                        If CLng(strTDADay) >= 1 And CLng(strTDADay) <= 29 Then
                            If strTDADay = "29" Then
                                If Len(strTYER) > 0 Then
                                    If CLng(strTYER) / 4 = Int(CLng(strTYER) / 4) And Not (CLng(strTYER) / 100 = Int(CLng(strTYER) / 100) And Not (CLng(strTYER) / 400 = Int(CLng(strTYER) / 400))) Then
                                        strTDAT = strTDAData
                                    Else
                                        strTDAT = ""
                                    End If
                                Else
                                    strTDAT = strTDAData
                                End If
                            Else
                                strTDAT = strTDAData
                            End If
                        Else
                            strTDAT = ""
                        End If
                    Case Else
                        strTDAT = ""
                End Select
            Else
                strTDAT = ""
            End If
        Case "TIM" 'Time
            strTIMData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If Len(strTIMData) = 4 And IsNumeric(strTIMData) Then
                lngTIMHour = CLng(Mid(strTIMData, 1, 2))
                lngTIMMinute = CLng(Mid(strTIMData, 3, 2))
                If lngTIMHour <= 23 And lngTIMMinute <= 59 Then
                    strTIME = strTIMData
                Else
                    strTIME = ""
                End If
            Else
                strTIME = ""
            End If
        Case "TRD" 'Recording dates
            strTRDA = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TMT" 'Media type
            strTMED = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TFT" 'File type
            strTFLT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TBP" 'BPM (Beats Per Minute)
            strTBPData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTBPData) Then
                strTBPM = CStr(CLng(strTBPData))
            Else
                strTBPM = ""
            End If
        Case "TCR" 'Copyright message
            strTCRData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(Mid(strTCRData, 1, 4)) And Mid(strTCRData, 5, 1) = " " Then
                strTCOP = strTCRData
            Else
                strTCOP = ""
            End If
        Case "TPB" 'Publisher
            strTPUB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TEN" 'Encoded by
            strTENC = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSS" 'Software/hardware and settings used for encoding
            strTSSE = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOF" 'Original filename
            strTOFN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TLE" 'Length
            strTLEData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTLEData) Then
                strTLEN = CStr(CLng(strTLEData))
            Else
                strTLEN = ""
            End If
        Case "TSI" 'Size
            strTSIData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTSIData) Then
                strTSIZ = CStr(CLng(strTSIData))
            Else
                strTSIZ = ""
            End If
        Case "TDY" 'Playlist delay
            strTDYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTDYData) Then
                strTDLY = CStr(CLng(strTDYData))
            Else
                strTDLY = ""
            End If
        Case "TKE" 'Initial key
            strTKEData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTKEData)
                Case 1
                    If InStr("ABCDEFGo", strTKEData) > 0 Then
                        strTKEY = strTKEData
                    Else
                        strTKEY = ""
                    End If
                Case 2
                    If InStr("ABCDEFG", Mid(strTKEData, 1, 1)) > 0 Then
                        If InStr("b#m", Mid(strTKEData, 2, 1)) > 0 Then
                            If strTKEData = "E#" Or strTKEData = "Fb" Or strTKEData = "B#" Or strTKEData = "Cb" Then
                                strTKEY = ""
                            Else
                                strTKEY = strTKEData
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case 3
                    If InStr("ABCDEFG", Mid(strTKEData, 1, 1)) > 0 Then
                        If InStr("b#", Mid(strTKEData, 2, 1)) > 0 Then
                            If Mid(strTKEData, 1, 2) = "E#" Or Mid(strTKEData, 1, 2) = "Fb" Or Mid(strTKEData, 1, 2) = "B#" Or Mid(strTKEData, 1, 2) = "Cb" Then
                                strTKEY = ""
                            Else
                                If Mid(strTKEData, 3, 1) = "m" Then
                                    strTKEY = strTKEData
                                Else
                                    strTKEY = ""
                                End If
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case Else
                    strTKEY = ""
            End Select
        Case "TOT" 'Original album/Movie/Show title
            strTOAL = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOA" 'Original artist(s)/performer(s)
            strTOPE = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOL" 'Original Lyricist(s)/text writer(s)
            strTOLY = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOR" 'Original release year
            strTORData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTORData) Then
                strTORY = CStr(CLng(strTORData))
            Else
                strTORY = ""
            End If
        Case "TXX" 'User defined text information
            strTXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strTXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strTXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strTXXX = strTXXX & strTXXData & Chr(0)
                End If
            End If
        Case "WAF" 'Official audio file webpage
            strWOAF = strTagData
        Case "WAR" 'Official artist/performer webpage
            strWOAR = strWOAR & strTagData & Chr(0)
        Case "WAS" 'Official audio source webpage
            strWOAS = strTagData
        Case "WCM" 'Commercial information
            strWCOM = strWCOM & strTagData & Chr(0)
        Case "WCP" 'Copyright/Legal information
            strWCOP = strTagData
        Case "WPB" 'Publishers official webpage
            strWPUB = strTagData
        Case "WXX" 'User defined URL link
            strWXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strWXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strWXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strWXXX = strWXXX & strWXXData & Chr(0)
                End If
            End If
        Case "COM" 'Comments
            strLanguage = Mid(strTagData, 2, 4)
            If LCase(strLanguage) = "eng" Then
                strCOMData = Decode(Mid(strTagData, 5, Len(strTagData) - 4), Asc(Mid(strTagData, 1, 1)))
                lngNullTerm = InStr(strCOMData, Chr(0))
                If lngNullTerm > 1 Then
                    lngAnotherNullTerm = InStr(lngNullTerm + 1, strCOMData, Chr(0))
                    If lngAnotherNullTerm = 0 Then
                        strCOMM = strCOMM & strCOMData & Chr(0)
                    End If
                End If
            End If
    End Select
   
End Sub

'ProcessID3v2_3Tag - Processes an ID3v2.3 tag
Private Sub ProcessID3v2_3Tag(strID3v2Tag As String, strTagData As String)
    Dim lngNullTerm As Long, lngAnotherNullTerm As Long, lngCloseParen As Long, lngInParenVal As Long, lngTIMEHour As Long, lngTIMEMinute As Long, lngFoundSlash As Long
    Dim strTBPMData As String, strTCONData As String, strTCONData2 As String, strInParen As String, strTCOPData As String, strTDATData As String, strTDATDay As String, strTDATMonth As String, strTDLYData As String, strTIMEData As String, strTKEYData As String, strTLENData As String, strTORYData As String, strTPE1Data As String, strTPOSData As String, strTRCKData As String, strTSIZData As String, strTSRCData As String, strCountryCode As String, strRegistrantCode As String, strYearOfReference As String, strDesignationCode As String, strTYERData As String, strTXXXData As String, strWXXXData As String, strLanguage As String, strCOMMData As String
    Dim blnValidTSRC As Boolean
    Dim intCheckCount As Integer
    
    Select Case Mid(strID3v2Tag, 1, 4)
        Case "UFID" 'Unique File Identifier
            If Asc(Mid(strTagData, 1, 1)) > 0 Then
                lngNullTerm = InStr(strTagData, Chr(0))
                If lngNullTerm > 0 Then
                    strUFIDOwner = Mid(strTagData, 1, lngNullTerm - 1)
                    strUFIDIdentifier = NullTruncate(Mid(strTagData, lngNullTerm + 1, Len(strTagData) - lngNullTerm))
                End If
            End If
        Case "TALB" 'Album/Movie/Show title
            strTALB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strAlbum = strTALB
        Case "TBPM" 'BPM (Beats Per Minute)
            strTBPMData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTBPMData) Then
                strTBPM = CStr(CLng(strTBPMData))
            Else
                strTBPM = ""
            End If
        Case "TCOM" 'Composer(s)
            strTCOM = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TCON" 'Content type
            strTCONData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTCONData2 = ""
            Do While Len(strTCONData) > 0
                If Mid(strTCONData, 1, 1) = "(" Then
                    If Mid(strTCONData, 2, 1) = "(" Then
                        strTCONData2 = strTCONData2 & Mid(strTCONData, 2, Len(strTCONData) - 1) & Chr(0)
                        strTCONData = ""
                    Else
                        lngCloseParen = InStr(strTCONData, ")")
                        If lngCloseParen > 2 Then
                            strInParen = Mid(strTCONData, 2, lngCloseParen - 2)
                            If strInParen = "RX" Or strInParen = "CR" Then
                                strTCONData2 = strTCONData2 & strInParen & Chr(0)
                                strTCONData = Mid(strTCONData, lngCloseParen + 1, Len(strTCONData) - lngCloseParen)
                            Else
                                If IsNumeric(strInParen) Then
                                    lngInParenVal = CLng(strInParen)
                                    If lngInParenVal >= 0 And lngInParenVal <= 147 Then
                                        strTCONData2 = strTCONData2 & CStr(lngInParenVal) & Chr(0)
                                        strTCONData = Mid(strTCONData, lngCloseParen + 1, Len(strTCONData) - lngCloseParen)
                                    Else
                                        strTCONData2 = strTCONData2 & strTCONData & Chr(0)
                                        strTCONData = ""
                                    End If
                                Else
                                    strTCONData2 = strTCONData2 & strTCONData & Chr(0)
                                    strTCONData = ""
                                End If
                            End If
                        Else
                            strTCONData2 = strTCONData2 & strTCONData & Chr(0)
                            strTCONData = ""
                        End If
                    End If
                Else
                    strTCONData2 = strTCONData2 & strTCONData & Chr(0)
                    strTCONData = ""
                End If
            Loop
            If Len(strTCONData2) > 0 Then
                strTCONData2 = Mid(strTCONData2, 1, Len(strTCONData2) - 1)
                strTCON = Split(strTCONData2, Chr(0))
                If IsNumeric(strTCON(0)) Then
                    lngGenreID = CLng(strTCON(0))
                End If
            End If
        Case "TCOP" 'Copyright message
            strTCOPData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(Mid(strTCOPData, 1, 4)) And Mid(strTCOPData, 5, 1) = " " Then
                strTCOP = strTCOPData
            Else
                strTCOP = ""
            End If
        Case "TDAT" 'Date
            strTDATData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If Len(strTDATData) = 4 And IsNumeric(strTDATData) Then
                strTDATDay = Mid(strTDATData, 1, 2)
                strTDATMonth = Mid(strTDATData, 3, 2)
                Select Case strTDATMonth
                    Case "01", "03", "05", "07", "08", "10", "12"
                        If CLng(strTDATDay) >= 1 And CLng(strTDATDay) <= 31 Then
                            strTDAT = strTDATData
                        Else
                            strTDAT = ""
                        End If
                    Case "04", "06", "09", "11"
                        If CLng(strTDATDay) >= 1 And CLng(strTDATDay) <= 30 Then
                            strTDAT = strTDATData
                        Else
                            strTDAT = ""
                        End If
                    Case "02"
                        If CLng(strTDATDay) >= 1 And CLng(strTDATDay) <= 29 Then
                            If strTDATDay = "29" Then
                                If Len(strTYER) > 0 Then
                                    If CLng(strTYER) / 4 = Int(CLng(strTYER) / 4) And Not (CLng(strTYER) / 100 = Int(CLng(strTYER) / 100) And Not (CLng(strTYER) / 400 = Int(CLng(strTYER) / 400))) Then
                                        strTDAT = strTDATData
                                    Else
                                        strTDAT = ""
                                    End If
                                Else
                                    strTDAT = strTDATData
                                End If
                            Else
                                strTDAT = strTDATData
                            End If
                        Else
                            strTDAT = ""
                        End If
                    Case Else
                        strTDAT = ""
                End Select
            Else
                strTDAT = ""
            End If
        Case "TDLY" 'Playlist delay
            strTDLYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTDLYData) Then
                strTDLY = CStr(CLng(strTDLYData))
            Else
                strTDLY = ""
            End If
        Case "TENC" 'Encoded by
            strTENC = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TEXT" 'Lyricist(s)/text writer(s)
            strTEXT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TFLT" 'File type
            strTFLT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TIME" 'Time
            strTIMEData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If Len(strTIMEData) = 4 And IsNumeric(strTIMEData) Then
                lngTIMEHour = CLng(Mid(strTIMEData, 1, 2))
                lngTIMEMinute = CLng(Mid(strTIMEData, 3, 2))
                If lngTIMEHour <= 23 And lngTIMEMinute <= 59 Then
                    strTIME = strTIMEData
                Else
                    strTIME = ""
                End If
            Else
                strTIME = ""
            End If
        Case "TIT1" 'Content group description
            strTIT1 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TIT2" 'Title/Songname/Content description
            strTIT2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTitle = strTIT2
        Case "TIT3" 'Subtitle/Description refinement
            strTIT3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TKEY" 'Initial key
            strTKEYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTKEYData)
                Case 1
                    If InStr("ABCDEFGo", strTKEYData) > 0 Then
                        strTKEY = strTKEYData
                    Else
                        strTKEY = ""
                    End If
                Case 2
                    If InStr("ABCDEFG", Mid(strTKEYData, 1, 1)) > 0 Then
                        If InStr("b#m", Mid(strTKEYData, 2, 1)) > 0 Then
                            If strTKEYData = "E#" Or strTKEYData = "Fb" Or strTKEYData = "B#" Or strTKEYData = "Cb" Then
                                strTKEY = ""
                            Else
                                strTKEY = strTKEYData
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case 3
                    If InStr("ABCDEFG", Mid(strTKEYData, 1, 1)) > 0 Then
                        If InStr("b#", Mid(strTKEYData, 2, 1)) > 0 Then
                            If Mid(strTKEYData, 1, 2) = "E#" Or Mid(strTKEYData, 1, 2) = "Fb" Or Mid(strTKEYData, 1, 2) = "B#" Or Mid(strTKEYData, 1, 2) = "Cb" Then
                                strTKEY = ""
                            Else
                                If Mid(strTKEYData, 3, 1) = "m" Then
                                    strTKEY = strTKEYData
                                Else
                                    strTKEY = ""
                                End If
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case Else
                    strTKEY = ""
            End Select
        Case "TLAN" 'Language(s)
            strTLAN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TLEN" 'Length
            strTLENData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTLENData) Then
                strTLEN = CStr(CLng(strTLENData))
            Else
                strTLEN = ""
            End If
        Case "TMED" 'Media type
            strTMED = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOAL" 'Original album/Movie/Show title
            strTOAL = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOFN" 'Original filename
            strTOFN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOLY" 'Original Lyricist(s)/text writer(s)
            strTOLY = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOPE" 'Original artist(s)/performer(s)
            strTOPE = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TORY" 'Original release year
            strTORYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTORYData) Then
                strTORY = CStr(CLng(strTORYData))
            Else
                strTORY = ""
            End If
        Case "TOWN" 'File owner/licensee
            strTOWN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPE1" 'Lead artist(s)/Lead performer(s)/Soloist(s)/Performing group
            strTPE1Data = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTPE1 = Split(strTPE1Data, "/")
            strArtist = strTPE1Data
        Case "TPE2" 'Band/Orchestra/Accompaniment
            strTPE2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPE3" 'Conductor
            strTPE3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPE4" 'Interpreted, remixed, or otherwise modified by
            strTPE4 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPOS" 'Part of a set
            strTPOSData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTPOSData) Then
                strTPOS = CStr(CLng(strTPOSData))
            Else
                lngFoundSlash = InStr(strTPOSData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTPOSData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTPOSData, lngFoundSlash + 1, Len(strTPOSData) - lngFoundSlash)) Then
                        strTPOS = CStr(CLng(Mid(strTPOSData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTPOSData, lngFoundSlash + 1, Len(strTPOSData) - lngFoundSlash)))
                    Else
                        strTPOS = ""
                    End If
                Else
                    strTPOS = ""
                End If
            End If
        Case "TPUB" 'Publisher
            strTPUB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TRCK" 'Track number/Position in set
            strTRCKData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTRCKData) Then
                strTRCK = CStr(CLng(strTRCKData))
                strTrack = strTRCK
            Else
                lngFoundSlash = InStr(strTRCKData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTRCKData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTRCKData, lngFoundSlash + 1, Len(strTRCKData) - lngFoundSlash)) Then
                        strTRCK = CStr(CLng(Mid(strTRCKData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTRCKData, lngFoundSlash + 1, Len(strTRCKData) - lngFoundSlash)))
                        strTrack = CStr(CLng(Mid(strTRCKData, 1, lngFoundSlash - 1)))
                    Else
                        strTRCK = ""
                    End If
                Else
                    strTRCK = ""
                End If
            End If
        Case "TRDA" 'Recording dates
            strTRDA = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TRSO" 'Internet radio station owner
            strTRSO = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSIZ" 'Size
            strTSIZData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTSIZData) Then
                strTSIZ = CStr(CLng(strTSIZData))
            Else
                strTSIZ = ""
            End If
        Case "TSRC" 'ISRC
            strTSRCData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTSRCData)
                Case 12
                    strCountryCode = UCase(Mid(strTSRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 3, 3))
                    strYearOfReference = Mid(strTSRCData, 6, 2)
                    strDesignationCode = Mid(strTSRCData, 8, 5)
                Case 15
                    strCountryCode = UCase(Mid(strTSRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 4, 3))
                    strYearOfReference = Mid(strTSRCData, 8, 2)
                    strDesignationCode = Mid(strTSRCData, 11, 5)
                Case 16
                    strCountryCode = UCase(Mid(strTSRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 7, 3))
                    strYearOfReference = Mid(strTSRCData, 10, 2)
                    strDesignationCode = Mid(strTSRCData, 12, 5)
                Case 19
                    strCountryCode = UCase(Mid(strTSRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 8, 3))
                    strYearOfReference = Mid(strTSRCData, 12, 2)
                    strDesignationCode = Mid(strTSRCData, 15, 5)
                Case Else
                    strCountryCode = "AB"
                    strRegistrantCode = "ABC"
                    strYearOfReference = "AB"
                    strDesignationCode = "ABCDE"
            End Select
            If Not (IsNumeric(strYearOfReference) And IsNumeric(strDesignationCode)) Then
                strTSRC = ""
            Else
                blnValidTSRC = True
                For intCheckCount = 1 To 3
                    If intCheckCount < 3 Then
                        If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strCountryCode, intCheckCount, 1)) = 0 Then
                            blnValidTSRC = False
                            Exit For
                        End If
                    End If
                Next
                If blnValidTSRC Then
                    strTSRC = strCountryCode & strRegistrantCode & strYearOfReference & strDesignationCode
                Else
                    strTSRC = ""
                End If
            End If
        Case "TSSE" 'Software/hardware and settings used for encoding
            strTSSE = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TYER" 'Year
            strTYERData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTYERData) Then
                strTYER = CStr(CLng(strTYERData))
            Else
                strTYER = ""
            End If
            strSongYear = strTYER
        Case "TXXX" 'User defined text information
            strTXXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strTXXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strTXXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strTXXX = strTXXX & strTXXXData & Chr(0)
                End If
            End If
        Case "WCOM" 'Commercial information
            strWCOM = strWCOM & strTagData & Chr(0)
        Case "WCOP" 'Copyright/Legal information
            strWCOP = strTagData
        Case "WOAF" 'Official audio file webpage
            strWOAF = strTagData
        Case "WOAR" 'Official artist/performer webpage
            strWOAR = strWOAR & strTagData & Chr(0)
        Case "WOAS" 'Official audio source webpage
            strWOAS = strTagData
        Case "WORS" 'Official internet radio station homepage
            strWORS = strTagData
        Case "WPAY" 'Payment
            strWPAY = strTagData
        Case "WPUB" 'Publishers official webpage
            strWPUB = strTagData
        Case "WXXX" 'User defined URL link
            strWXXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strWXXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strWXXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strWXXX = strWXXX & strWXXXData & Chr(0)
                End If
            End If
        Case "COMM" 'Comments
            strLanguage = Mid(strTagData, 2, 3)
            If LCase(strLanguage) = "eng" Then
                strCOMMData = Decode(Mid(strTagData, 5, Len(strTagData) - 4), Asc(Mid(strTagData, 1, 1)))
                lngNullTerm = InStr(strCOMMData, Chr(0))
                If lngNullTerm > 1 Then
                    lngAnotherNullTerm = InStr(lngNullTerm + 1, strCOMMData, Chr(0))
                    If lngAnotherNullTerm = 0 Then
                        strCOMM = strCOMM & strCOMMData & Chr(0)
                    End If
                End If
            End If
    End Select
End Sub

'ProcessID3v2_4Tag - Processes an ID3v2.4 tag
Private Sub ProcessID3v2_4Tag(strID3v2Tag As String, strTagData As String)
    Dim lngNullTerm As Long, lngFoundSlash As Long, lngAnotherNullTerm As Long
    Dim strTRCKData As String, strTPOSData As String, strTSRCData As String, strCountryCode As String, strRegistrantCode As String, strYearOfReference As String, strDesignationCode As String, strTPE1Data As String, strTOPEData As String, strTOLYData As String, strTMCLData As String, strTBPMData As String, strTLENData As String, strTKEYData As String, strTCONData As String, strTCOPData As String, strTPROData As String, strTDLYData As String, strTDENData As String, strTDORData As String, strTDRCData As String, strTDRLData As String, strTDTGData As String, strTXXXData As String, strWXXXData As String, strLanguage As String, strCOMMData As String
    Dim blnValidTSRC As Boolean
    Dim intCheckCount As Integer
    
    Select Case Mid(strID3v2Tag, 1, 4)
        Case "UFID" 'Unique File Identifier
            If Asc(Mid(strTagData, 1, 1)) > 0 Then
                lngNullTerm = InStr(strTagData, Chr(0))
                If lngNullTerm > 0 Then
                    strUFIDOwner = Mid(strTagData, 1, lngNullTerm - 1)
                    strUFIDIdentifier = NullTruncate(Mid(strTagData, lngNullTerm + 1, Len(strTagData) - lngNullTerm))
                End If
            End If
        Case "TIT1" 'Content group description
            strTIT1 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TIT2" 'Title/Songname/Content description
            strTIT2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strTitle = strTIT2
        Case "TIT3" 'Subtitle/Description refinement
            strTIT3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TALB" 'Album/Movie/Show title
            strTALB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            strAlbum = strTALB
        Case "TOAL" 'Original album/Movie/Show title
            strTOAL = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TRCK" 'Track number/Position in set
            strTRCKData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTRCKData) Then
                strTRCK = CStr(CLng(strTRCKData))
            Else
                lngFoundSlash = InStr(strTRCKData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTRCKData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTRCKData, lngFoundSlash + 1, Len(strTRCKData) - lngFoundSlash)) Then
                        strTRCK = CStr(CLng(Mid(strTRCKData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTRCKData, lngFoundSlash + 1, Len(strTRCKData) - lngFoundSlash)))
                    Else
                        strTRCK = ""
                    End If
                Else
                    strTRCK = ""
                End If
            End If
        Case "TPOS" 'Part of a set
            strTPOSData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTPOSData) Then
                strTPOS = CStr(CLng(strTPOSData))
            Else
                lngFoundSlash = InStr(strTPOSData, "/")
                If lngFoundSlash > 1 Then
                    If IsNumeric(Mid(strTPOSData, 1, lngFoundSlash - 1)) And IsNumeric(Mid(strTPOSData, lngFoundSlash + 1, Len(strTPOSData) - lngFoundSlash)) Then
                        strTPOS = CStr(CLng(Mid(strTPOSData, 1, lngFoundSlash - 1))) & "/" & CStr(CLng(Mid(strTPOSData, lngFoundSlash + 1, Len(strTPOSData) - lngFoundSlash)))
                    Else
                        strTPOS = ""
                    End If
                Else
                    strTPOS = ""
                End If
            End If
        Case "TSST" 'Set subtitle
            strTSST = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSRC" 'ISRC
            strTSRCData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTSRCData)
                Case 12
                    strCountryCode = UCase(Mid(strTSRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 3, 3))
                    strYearOfReference = Mid(strTSRCData, 6, 2)
                    strDesignationCode = Mid(strTSRCData, 8, 5)
                Case 15
                    strCountryCode = UCase(Mid(strTSRCData, 1, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 4, 3))
                    strYearOfReference = Mid(strTSRCData, 8, 2)
                    strDesignationCode = Mid(strTSRCData, 11, 5)
                Case 16
                    strCountryCode = UCase(Mid(strTSRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 7, 3))
                    strYearOfReference = Mid(strTSRCData, 10, 2)
                    strDesignationCode = Mid(strTSRCData, 12, 5)
                Case 19
                    strCountryCode = UCase(Mid(strTSRCData, 5, 2))
                    strRegistrantCode = UCase(Mid(strTSRCData, 8, 3))
                    strYearOfReference = Mid(strTSRCData, 12, 2)
                    strDesignationCode = Mid(strTSRCData, 15, 5)
                Case Else
                    strCountryCode = "AB"
                    strRegistrantCode = "ABC"
                    strYearOfReference = "AB"
                    strDesignationCode = "ABCDE"
            End Select
            If Not (IsNumeric(strYearOfReference) And IsNumeric(strDesignationCode)) Then
                strTSRC = ""
            Else
                blnValidTSRC = True
                For intCheckCount = 1 To 3
                    If intCheckCount < 3 Then
                        If InStr("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", Mid(strCountryCode, intCheckCount, 1)) = 0 Then
                            blnValidTSRC = False
                            Exit For
                        End If
                    End If
                Next
                If blnValidTSRC Then
                    strTSRC = strCountryCode & strRegistrantCode & strYearOfReference & strDesignationCode
                Else
                    strTSRC = ""
                End If
            End If
        Case "TPE1" 'Lead artist(s)/Lead performer(s)/Soloist(s)/Performing group
            strTPE1Data = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            strTPE1 = Split(strTPE1Data, Chr(0))
            strArtist = Replace(strTPE1Data, Chr(0), "/")
        Case "TPE2" 'Band/Orchestra/Accompaniment
            strTPE2 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPE3" 'Conductor
            strTPE3 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TPE4" 'Interpreted, remixed, or otherwise modified by
            strTPE4 = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOPE" 'Original artist(s)/performer(s)
            strTOPE = Replace(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))), Chr(0), "/")
        Case "TEXT" 'Lyricist(s)/text writer(s)
            strTEXT = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
        Case "TOLY" 'Original Lyricist(s)/text writer(s)
            strTOLY = Replace(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))), Chr(0), "/")
        Case "TCOM" 'Composer(s)
            strTCOM = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
        Case "TMCL" 'Musician credits list
            strTMCL = Split(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))), Chr(0))
        Case "TIPL" 'Involved people list
            strTIPL = Split(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))), Chr(0))
        Case "TENC" 'Encoded by
            strTENC = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TBPM" 'BPM (Beats Per Minute)
            strTBPMData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTBPMData) Then
                strTBPM = CStr(CLng(strTBPMData))
            Else
                strTBPM = ""
            End If
        Case "TLEN" 'Length
            strTLENData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTLENData) Then
                strTLEN = CStr(CLng(strTLENData))
            Else
                strTLEN = ""
            End If
        Case "TKEY" 'Initial key
            strTKEYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTKEYData)
                Case 1
                    If InStr("ABCDEFGo", strTKEYData) > 0 Then
                        strTKEY = strTKEYData
                    Else
                        strTKEY = ""
                    End If
                Case 2
                    If InStr("ABCDEFG", Mid(strTKEYData, 1, 1)) > 0 Then
                        If InStr("b#m", Mid(strTKEYData, 2, 1)) > 0 Then
                            If strTKEYData = "E#" Or strTKEYData = "Fb" Or strTKEYData = "B#" Or strTKEYData = "Cb" Then
                                strTKEY = ""
                            Else
                                strTKEY = strTKEYData
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case 3
                    If InStr("ABCDEFG", Mid(strTKEYData, 1, 1)) > 0 Then
                        If InStr("b#", Mid(strTKEYData, 2, 1)) > 0 Then
                            If Mid(strTKEYData, 1, 2) = "E#" Or Mid(strTKEYData, 1, 2) = "Fb" Or Mid(strTKEYData, 1, 2) = "B#" Or Mid(strTKEYData, 1, 2) = "Cb" Then
                                strTKEY = ""
                            Else
                                If Mid(strTKEYData, 3, 1) = "m" Then
                                    strTKEY = strTKEYData
                                Else
                                    strTKEY = ""
                                End If
                            End If
                        Else
                            strTKEY = ""
                        End If
                    Else
                        strTKEY = ""
                    End If
                Case Else
                    strTKEY = ""
            End Select
        Case "TLAN" 'Language(s)
            strTLAN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TCON" 'Content type
            strTCONData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            strTCON = Split(strTCONData, Chr(0))
            If IsNumeric(strTCON(0)) Then
                lngGenreID = CLng(strTCON(0))
            End If
        Case "TFLT" 'File type
            strTFLT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TMED" 'Media type
            strTMED = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
        Case "TMOO" 'Mood
            strTMOO = Split(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))), Chr(0))
        Case "TCOP" 'Copyright message
            strTCOPData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(Mid(strTCOPData, 1, 4)) And Mid(strTCOPData, 5, 1) = " " Then
                strTCOP = strTCOPData
            Else
                strTCOP = ""
            End If
        Case "TPRO" 'Produced notice
            strTPROData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(Mid(strTPROData, 1, 4)) And Mid(strTPROData, 5, 1) = " " Then
                strTPRO = strTPROData
            Else
                strTPRO = ""
            End If
        Case "TPUB" 'Publisher
            strTPUB = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOWN" 'File owner/licensee
            strTOWN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TRSN" 'Internet radio station name
            strTRSN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TRSO" 'Internet radio station owner
            strTRSO = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TOFN" 'Original filename
            strTOFN = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TDLY" 'Playlist delay
            strTDLYData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            If IsNumeric(strTDLYData) Then
                strTDLY = CStr(CLng(strTDLYData))
            Else
                strTDLY = ""
            End If
        Case "TDEN" 'Encoding time
            strTDENData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTDENData)
                Case 4
                    If IsNumeric(strTDENData) Then
                        strTDEN = strTDENData
                    Else
                        strTDEN = ""
                    End If
                Case 7, 10
                    If IsDate(strTDENData) Then
                        strTDEN = strTDENData
                    Else
                        strTDEN = ""
                    End If
                Case 13
                    strTDENData = Replace(strTDENData, "T", " ")
                    strTDENData = strTDENData & ":00"
                    If IsDate(strTDENData) Then
                        strTDEN = strTDENData
                    Else
                        strTDEN = ""
                    End If
                Case 16, 19
                    strTDENData = Replace(strTDENData, "T", " ")
                    If IsDate(strTDENData) Then
                        strTDEN = strTDENData
                    Else
                        strTDEN = ""
                    End If
            End Select
        Case "TDOR" 'Original release time
            strTDORData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTDORData)
                Case 4
                    If IsNumeric(strTDORData) Then
                        strTDOR = strTDORData
                    Else
                        strTDOR = ""
                    End If
                Case 7, 10
                    If IsDate(strTDORData) Then
                        strTDOR = strTDORData
                    Else
                        strTDOR = ""
                    End If
                Case 13
                    strTDORData = Replace(strTDORData, "T", " ")
                    strTDORData = strTDORData & ":00"
                    If IsDate(strTDORData) Then
                        strTDOR = strTDORData
                    Else
                        strTDOR = ""
                    End If
                Case 16, 19
                    strTDORData = Replace(strTDORData, "T", " ")
                    If IsDate(strTDORData) Then
                        strTDOR = strTDORData
                    Else
                        strTDOR = ""
                    End If
            End Select
        Case "TDRC" 'Recording time
            strTDRCData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTDRCData)
                Case 4
                    If IsNumeric(strTDRCData) Then
                        strTDRC = strTDRCData
                    Else
                        strTDRC = ""
                    End If
                Case 7, 10
                    If IsDate(strTDRCData) Then
                        strTDRC = strTDRCData
                    Else
                        strTDRC = ""
                    End If
                Case 13
                    strTDRCData = Replace(strTDRCData, "T", " ")
                    strTDRCData = strTDRCData & ":00"
                    If IsDate(strTDRCData) Then
                        strTDRC = strTDRCData
                    Else
                        strTDRC = ""
                    End If
                Case 16, 19
                    strTDRCData = Replace(strTDRCData, "T", " ")
                    If IsDate(strTDRCData) Then
                        strTDRC = strTDRCData
                    Else
                        strTDRC = ""
                    End If
            End Select
        Case "TDRL" 'Release time
            strTDRLData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTDRLData)
                Case 4
                    If IsNumeric(strTDRLData) Then
                        strTDRL = strTDRLData
                        strTYER = strTDRL
                    Else
                        strTDRL = ""
                    End If
                Case 7, 10
                    If IsDate(strTDRLData) Then
                        strTDRL = strTDRLData
                        strTYER = Mid(strTDRL, 1, 4)
                        If Len(strTDRLData) = 10 Then
                            strTDAT = Mid(strTDRLData, 6, 2) & Mid(strTDRLData, 9, 2)
                        End If
                    Else
                        strTDRL = ""
                    End If
                Case 13
                    strTDRLData = Replace(strTDRLData, "T", " ")
                    strTDRLData = strTDRLData & ":00"
                    If IsDate(strTDRLData) Then
                        strTDRL = strTDRLData
                        strTYER = Mid(strTDRL, 1, 4)
                        strTDAT = Mid(strTDRLData, 6, 2) & Mid(strTDRLData, 9, 2)
                        strTIME = Mid(strTDRLData, 12, 2) & Mid(strTDRLData, 15, 2)
                    Else
                        strTDRL = ""
                    End If
                Case 16, 19
                    strTDRLData = Replace(strTDRLData, "T", " ")
                    If IsDate(strTDRLData) Then
                        strTDRL = strTDRLData
                        strTYER = Mid(strTDRL, 1, 4)
                        strTDAT = Mid(strTDRLData, 6, 2) & Mid(strTDRLData, 9, 2)
                        strTIME = Mid(strTDRLData, 12, 2) & Mid(strTDRLData, 15, 2)
                    Else
                        strTDRL = ""
                    End If
            End Select
            If Len(strTDRL) >= 4 Then
                strTYER = CStr(CLng(Mid(strTDRL, 1, 4)))
            End If
        Case "TDTG" 'Tagging time
            strTDTGData = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
            Select Case Len(strTDTGData)
                Case 4
                    If IsNumeric(strTDTGData) Then
                        strTDTG = strTDTGData
                    Else
                        strTDTG = ""
                    End If
                Case 7, 10
                    If IsDate(strTDTGData) Then
                        strTDTG = strTDTGData
                    Else
                        strTDTG = ""
                    End If
                Case 13
                    strTDTGData = Replace(strTDTGData, "T", " ")
                    strTDTGData = strTDTGData & ":00"
                    If IsDate(strTDTGData) Then
                        strTDTG = strTDTGData
                    Else
                        strTDTG = ""
                    End If
                Case 16, 19
                    strTDTGData = Replace(strTDTGData, "T", " ")
                    If IsDate(strTDTGData) Then
                        strTDTG = strTDTGData
                    Else
                        strTDTG = ""
                    End If
            End Select
        Case "TSSE" 'Software/hardware and settings used for encoding
            strTSSE = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSOA" 'Album sort order
            strTSOA = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSOP" 'Performer sort order
            strTSOP = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TSOT" 'Title sort order
            strTSOT = NullTruncate(Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1))))
        Case "TXXX" 'User defined text information
            strTXXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strTXXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strTXXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strTXXX = strTXXX & strTXXXData & Chr(0)
                End If
            End If
        Case "WCOM" 'Commercial information
            strWCOM = strWCOM & strTagData & Chr(0)
        Case "WCOP" 'Copyright/Legal information
            strWCOP = strTagData
        Case "WOAF" 'Official audio file webpage
            strWOAF = strTagData
        Case "WOAR" 'Official artist/performer webpage
            strWOAR = strWOAR & strTagData & Chr(0)
        Case "WOAS" 'Official audio source webpage
            strWOAS = strTagData
        Case "WORS" 'Official internet radio station homepage
            strWORS = strTagData
        Case "WPAY" 'Payment
            strWPAY = strTagData
        Case "WPUB" 'Publishers official webpage
            strWPUB = strTagData
        Case "WXXX" 'User defined URL link
            strWXXXData = Decode(Mid(strTagData, 2, Len(strTagData) - 1), Asc(Mid(strTagData, 1, 1)))
            lngNullTerm = InStr(strWXXXData, Chr(0))
            If lngNullTerm > 1 Then
                lngAnotherNullTerm = InStr(lngNullTerm + 1, strWXXXData, Chr(0))
                If lngAnotherNullTerm = 0 Then
                    strWXXX = strWXXX & strWXXXData & Chr(0)
                End If
            End If
        Case "COMM" 'Comments
            strLanguage = Mid(strTagData, 2, 3)
            If LCase(strLanguage) = "eng" Then
                strCOMMData = Decode(Mid(strTagData, 5, Len(strTagData) - 4), Asc(Mid(strTagData, 1, 1)))
                lngNullTerm = InStr(strCOMMData, Chr(0))
                If lngNullTerm > 1 Then
                    lngAnotherNullTerm = InStr(lngNullTerm + 1, strCOMMData, Chr(0))
                    If lngAnotherNullTerm = 0 Then
                        strCOMM = strCOMM & strCOMMData & Chr(0)
                    End If
                End If
            End If
    End Select
End Sub

'Decode - Function to decode a string based on the type of encoding it was given.
Private Function Decode(strData As String, intTextEncoding As Integer) As String
    If intTextEncoding = 1 Then
        Dim cvt As OleCvt
        Decode = cvt.DecodeUnicodeName(strData)
        cvt = Nothing
    Else
        Decode = strData
    End If
End Function
