VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsOGG"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'This file contains a class for detecting OGG files, as well
'as learning information about the file, and parsing any
'comments found within the file.  You can also write comments
'back to the file.

'oggfile - The main class, oggfile, contains all of the functions
'to extract data from the OGG file.

Option Explicit

Private strCommentsFields() As String
Private strCommentsValues() As String
Private intCommentsCount As Integer

Public strOGGFilename As String
Public lngFileSize As Long
Public lngSongLength As Long
Public blnIsOGG As Boolean
Public lngStreamSerialNumber As Long
Public intAudioChannels As Integer
Public lngSampleRate As Long
Public lngBitrateMaximum As Long
Public lngBitrateNominal As Long 'This is NOT the average bitrate!  This is simply the target bitrate for the encoder.
Public lngBitrateMinimum As Long
Public intAverageBitrate As Integer
Public strVendorString As String
Public strSetupHeader As String
Public lngAudioPositionStart As Long
Public blnIsVBR As Boolean

'Initialization
Private Sub Class_Initialize()
    blnIsOGG = False
    intCommentsCount = 0
    ReDim strCommentsFields(0)
    ReDim strCommentsValues(0)
End Sub

'LoadOGG - Loads an OGG file passed as a path represented as a string.  Returns True if file passed is an OGG, False if not.
Public Function LoadOGG(strFilename As String) As Boolean
    Dim fso As New FileSystemObject
    Dim f As File
    Dim ts As TextStream
    Dim lngPageStart As Long, lngAudioBytes As Long
    Dim blnProcessedHeaders As Boolean
    Dim strHeaderString As String
    
    blnIsOGG = True
    LoadOGG = True

    'Get filesize.
    On Error Resume Next
    Set f = fso.GetFile(strFilename)
    On Error GoTo 0
    If Not IsObject(f) Then
        blnIsOGG = False
        LoadOGG = False
        GoTo EndFunction
    End If
    lngFileSize = f.Size
    Set f = Nothing
    
    'Read pages until the headers have been read.
    Set ts = fso.OpenTextFile(strFilename, 1)
    lngPageStart = 0
    blnProcessedHeaders = False
    strHeaderString = ""
    lngAudioBytes = 0
    Do
        If Not ParsePage(lngPageStart, ts, strHeaderString, blnProcessedHeaders, lngAudioBytes) Then
            blnIsOGG = False
            LoadOGG = False
            GoTo EndFunction
        End If
    Loop Until blnProcessedHeaders
    
    strOGGFilename = strFilename
    
EndFunction:
    ts.Close
    Set ts = Nothing
    Set f = Nothing
    Set fso = Nothing
End Function

'CalculateAverageBitrate - Calculates the average bitrate of the file.
Public Function CalculateAverageBitrate(strFilename As String) As Boolean
    Dim fso As New FileSystemObject
    Dim ts As TextStream
    Dim lngPageStart As Long, lngAudioBytes As Long
    Dim blnProcessedHeaders As Boolean
    Dim strHeaderString As String

    'Loop through each OGG page
    Set ts = fso.OpenTextFile(strFilename, 1)
    lngPageStart = 0
    blnProcessedHeaders = False
    strHeaderString = ""
    lngAudioBytes = 0
    Do
        If Not ParsePage(lngPageStart, ts, strHeaderString, blnProcessedHeaders, lngAudioBytes) Then
            blnIsOGG = False
            CalculateAverageBitrate = False
            GoTo EndFunction
        End If
    Loop Until lngPageStart >= lngFileSize - 25

    'We now have enough information to calculate the average bitrate
    intAverageBitrate = Int(8 * lngFileSize / lngSongLength)
    CalculateAverageBitrate = True
    
EndFunction:
    ts.Close
    Set ts = Nothing
    Set fso = Nothing
End Function

'ParsePage - Parses an individual page of an OGG file
Public Function ParsePage(ByRef lngPageStart As Long, ByRef ts As TextStream, ByRef strHeaderString As String, ByRef blnProcessedHeaders As Boolean, ByRef lngAudioBytes As Long) As Boolean
    Dim lngCurrentPosition As Long, lngPageSequenceNumber As Long, lngTotalSegmentSize As Long, lngBlocksize0 As Long, lngBlocksize1 As Long, lngVendorLength As Long, lngHeaderStart As Long, lngUserCommentListLength As Long, lngCommentCount As Long, lngCommentLength As Long, lngFoundEquals As Long
    Dim strPageCheck As String, strHeaderTypeFlag As String, strCommonHeader As String, strCommentPair As String, strFieldName As String, strFieldValue As String
    Dim intStreamStructureVersion As Integer, intPageSegments As Integer, intSegmentCount As Integer, intSegmentSize As Integer, intFrameFlag As Integer, intFramingBit As Integer
    Dim blnPacketFlag As Boolean, blnFirstPageFlag As Boolean, blnLastPageFlag As Boolean
    Dim dblAbsoluteGranulePosition As Double, dblPageChecksum As Double
    
    lngCurrentPosition = lngPageStart
    strPageCheck = ts.Read(27)
    lngPageStart = lngPageStart + 27

    'OGG page found.  Parse and skip it.
    If Mid(strPageCheck, 1, 4) = "OggS" Then

        'intStreamStructureVersion should be 0.  Select Case exists for future versions, if necessary.
        intStreamStructureVersion = Asc(Mid(strPageCheck, 5, 1))
        Select Case intStreamStructureVersion
            Case 0
                strHeaderTypeFlag = ToBin(Mid(strPageCheck, 6, 1))
                blnPacketFlag = CBool(Mid(strHeaderTypeFlag, 8, 1))
                blnFirstPageFlag = CBool(Mid(strHeaderTypeFlag, 7, 1))
                blnLastPageFlag = CBool(Mid(strHeaderTypeFlag, 6, 1))
                
                'Note: Because this is 8 bytes, and Visual Basic starts lopping off digits on anything more than 6, I am ignoring the last two characters here.
                '      I apologize to those who wish to handle 192KHz audio that is in excess of 23 years and 82 days in length, this script will not accomodate you. :)
                dblAbsoluteGranulePosition = Asc(Mid(strPageCheck, 7, 1)) + 256# * Asc(Mid(strPageCheck, 8, 1)) + 65536# * Asc(Mid(strPageCheck, 9, 1)) + 16777216# * Asc(Mid(strPageCheck, 10, 1)) + 4294967296# * Asc(Mid(strPageCheck, 11, 1)) + 1099511627776# * Asc(Mid(strPageCheck, 12, 1))
                If dblAbsoluteGranulePosition = 281474976710655# Then dblAbsoluteGranulePosition = -1

                'Get song length if we're on the last page and have a lngSampleRate
                If blnLastPageFlag And lngSampleRate > 0 Then
                    lngSongLength = dblAbsoluteGranulePosition / lngSampleRate
                End If

                lngStreamSerialNumber = Asc(Mid(strPageCheck, 15, 1)) + 256& * Asc(Mid(strPageCheck, 16, 1)) + 65536 * Asc(Mid(strPageCheck, 17, 1)) + 16777216 * Asc(Mid(strPageCheck, 18, 1))

                'I've found the Page Sequence Number to not always be accurate, so I don't process it at all.
                lngPageSequenceNumber = Asc(Mid(strPageCheck, 19, 1)) + 256& * Asc(Mid(strPageCheck, 20, 1)) + 65536 * Asc(Mid(strPageCheck, 21, 1)) + 16777216 * Asc(Mid(strPageCheck, 22, 1))

                dblPageChecksum = Asc(Mid(strPageCheck, 23, 1)) + 256# * Asc(Mid(strPageCheck, 24, 1)) + 65536# * Asc(Mid(strPageCheck, 25, 1)) + 16777216# * Asc(Mid(strPageCheck, 26, 1))

                'Get the number of segments, then read that many bytes
                intPageSegments = Asc(Mid(strPageCheck, 27, 1))
                strPageCheck = ts.Read(intPageSegments)
                lngPageStart = lngPageStart + intPageSegments

                'Determine the total segment size for this page
                lngTotalSegmentSize = 0
                For intSegmentCount = 1 To intPageSegments
                    intSegmentSize = Asc(Mid(strPageCheck, intSegmentCount, 1))
                    lngTotalSegmentSize = lngTotalSegmentSize + intSegmentSize
                Next

                'Check to see if this page contains headers, if so then read and add to the header string.
                If dblAbsoluteGranulePosition = 0 Then
                    strPageCheck = ts.Read(lngTotalSegmentSize)
                    lngPageStart = lngPageStart + lngTotalSegmentSize
                    strHeaderString = strHeaderString & strPageCheck
                Else 'No headers contained, there's actual audio data here
                    'If the headers haven't been processed yet, do so now.
                    If Not blnProcessedHeaders Then
                        lngAudioPositionStart = lngCurrentPosition
                        blnProcessedHeaders = True

                        'Read common header
                        strCommonHeader = Mid(strHeaderString, 1, 30)
                        If Mid(strCommonHeader, 1, 11) <> Chr(1) & "vorbis" & Chr(0) & Chr(0) & Chr(0) & Chr(0) Then
                            blnIsOGG = False
                            ParsePage = False
                            Exit Function
                        End If

                        intAudioChannels = Asc(Mid(strCommonHeader, 12, 1))

                        lngSampleRate = Asc(Mid(strCommonHeader, 13, 1)) + 256& * Asc(Mid(strCommonHeader, 14, 1)) + 65536 * Asc(Mid(strCommonHeader, 15, 1)) + 16777216 * Asc(Mid(strCommonHeader, 16, 1))

                        lngBitrateMaximum = Asc(Mid(strCommonHeader, 17, 1)) + 256& * Asc(Mid(strCommonHeader, 18, 1)) + 65536 * Asc(Mid(strCommonHeader, 19, 1)) + 16777216 * Asc(Mid(strCommonHeader, 20, 1))

                        lngBitrateNominal = Asc(Mid(strCommonHeader, 21, 1)) + 256& * Asc(Mid(strCommonHeader, 22, 1)) + 65536 * Asc(Mid(strCommonHeader, 23, 1)) + 16777216 * Asc(Mid(strCommonHeader, 24, 1))

                        lngBitrateMinimum = Asc(Mid(strCommonHeader, 25, 1)) + 256& * Asc(Mid(strCommonHeader, 26, 1)) + 65536 * Asc(Mid(strCommonHeader, 27, 1)) + 16777216 * Asc(Mid(strCommonHeader, 28, 1))

                        If lngBitrateMaximum <> lngBitrateMinimum Then blnIsVBR = True Else blnIsVBR = False

                        lngBlocksize0 = 2 ^ (Asc(Mid(strCommonHeader, 29, 1)) Mod 16)
                        lngBlocksize1 = 2 ^ (Asc(Mid(strCommonHeader, 29, 1)) \ 16)

                        intFrameFlag = Asc(Mid(strCommonHeader, 30, 1))

                        If intAudioChannels = 0 Or lngSampleRate = 0 Or lngBlocksize0 > lngBlocksize1 Or intFrameFlag <> 1 Then
                            blnIsOGG = False
                            ParsePage = False
                            Exit Function
                        End If

                        strHeaderString = Mid(strHeaderString, 31, Len(strHeaderString) - 30)
                        'Read comments header
                        If Mid(strHeaderString, 1, 7) <> Chr(3) & "vorbis" Then
                            blnIsOGG = False
                            ParsePage = False
                            Exit Function
                        End If

                        lngVendorLength = Asc(Mid(strHeaderString, 8, 1)) + 256& * Asc(Mid(strHeaderString, 9, 1)) + 65536 * Asc(Mid(strHeaderString, 10, 1)) + 16777216 * Asc(Mid(strHeaderString, 11, 1))

                        strVendorString = Mid(strHeaderString, 12, lngVendorLength)
                        lngHeaderStart = lngVendorLength + 12

                        lngUserCommentListLength = Asc(Mid(strHeaderString, lngHeaderStart, 1)) + 256& * Asc(Mid(strHeaderString, lngHeaderStart + 1, 1)) + 65536 * Asc(Mid(strHeaderString, lngHeaderStart + 2, 1)) + 16777216 * Asc(Mid(strHeaderString, lngHeaderStart + 3, 1))
                        lngHeaderStart = lngHeaderStart + 4

                        'Read user comments
                        For lngCommentCount = 1 To lngUserCommentListLength
                            lngCommentLength = Asc(Mid(strHeaderString, lngHeaderStart, 1)) + 256& * Asc(Mid(strHeaderString, lngHeaderStart + 1, 1)) + 65536 * Asc(Mid(strHeaderString, lngHeaderStart + 2, 1)) + 16777216 * Asc(Mid(strHeaderString, lngHeaderStart + 3, 1))
                            lngHeaderStart = lngHeaderStart + 4

                            strCommentPair = Mid(strHeaderString, lngHeaderStart, lngCommentLength)
                            lngHeaderStart = lngHeaderStart + lngCommentLength

                            'Read comment pair
                            lngFoundEquals = InStr(1, strCommentPair, "=")
                            strFieldName = Left(strCommentPair, lngFoundEquals - 1)
                            strFieldValue = Right(strCommentPair, Len(strCommentPair) - lngFoundEquals)

                            AddComment strFieldName, strFieldValue
                        Next

                        intFramingBit = Asc(Mid(strHeaderString, lngHeaderStart, 1))
                        lngHeaderStart = lngHeaderStart + 1
                        If intFramingBit <> 1 Then
                            blnIsOGG = False
                            ParsePage = False
                            Exit Function
                        End If

                        'At this point, we are done reading the comments header, all that remains is the setup header, which isn't going to contain very much useful information.
                        'However, we are going to save the setup header, in case we need to write it later.
                        strSetupHeader = Mid(strHeaderString, lngHeaderStart, Len(strHeaderString) - (lngHeaderStart - 1))
                    End If
                    ts.Skip lngTotalSegmentSize
                    lngPageStart = lngPageStart + lngTotalSegmentSize
                    lngAudioBytes = lngAudioBytes + lngTotalSegmentSize
                End If

            Case Else
                blnIsOGG = False
                ParsePage = False
                Exit Function
        End Select
    Else 'Otherwise, this is not an OGG page, and the file is likely corrupt.
        blnIsOGG = False
        ParsePage = False
        Exit Function
    End If
    
    ParsePage = True
End Function

'GetComment - Returns the comment value with the specified name.
Public Function GetComment(strFieldName As String) As String
    Dim lngCommentCount As Long
    
    For lngCommentCount = 1 To UBound(strCommentsFields)
        If UCase(strCommentsFields(lngCommentCount)) = UCase(strFieldName) Then
            GetComment = strCommentsValues(lngCommentCount)
            Exit Function
        End If
    Next
    GetComment = ""
End Function

'AddComment - Adds a comment to the comment collection with the specified name and value.
Public Sub AddComment(strFieldName As String, strFieldValue As String)
    Dim lngCommentCount As Long
    
    For lngCommentCount = 1 To UBound(strCommentsFields)
        If UCase(strCommentsFields(lngCommentCount)) = UCase(strFieldName) Then
            strCommentsValues(lngCommentCount) = strFieldValue
            Exit Sub
        End If
    Next
    intCommentsCount = intCommentsCount + 1
    ReDim Preserve strCommentsFields(intCommentsCount)
    ReDim Preserve strCommentsValues(intCommentsCount)
    strCommentsFields(intCommentsCount) = UCase(strFieldName)
    strCommentsValues(intCommentsCount) = strFieldValue
End Sub

'RemoveComment - Removes a comment from the comment collection with the specified name, preserving order of the remaining items.
Public Sub RemoveComment(strFieldName As String)
    Dim blnFoundComment As Boolean
    Dim lngCommentCount As Long
    
    blnFoundComment = False
    For lngCommentCount = 1 To UBound(strCommentsFields)
        If blnFoundComment Then
            strCommentsFields(lngCommentCount - 1) = strCommentsFields(lngCommentCount)
            strCommentsValues(lngCommentCount - 1) = strCommentsValues(lngCommentCount)
        End If
        If UCase(strCommentsFields(lngCommentCount)) = UCase(strFieldName) Then
            blnFoundComment = True
        End If
    Next
    If blnFoundComment Then
        intCommentsCount = intCommentsCount - 1
        ReDim Preserve strCommentsFields(intCommentsCount)
        ReDim Preserve strCommentsValues(intCommentsCount)
    End If
End Sub

'WriteOgg - Overwrites existing OGG file with new comments header.
Public Sub WriteOgg()
    Dim strCommentsHeader As String, strTwoHeaders As String, strHeaderPage As String, strHeaderPageTop As String, strHeaderPageBottom As String, strPageData As String
    Dim lngCommentCount As Long, lngLenStrCommentsHeader As Long, lngPageCount As Long
    Dim intPageSegments As Integer, intSegmentCount As Integer
    Dim crc As New clsCRC32
    Dim bytHeaderPage() As Byte, bytFirstHeaderPage() As Byte, bytNoData() As Byte, bytAudioPages() As Byte
    Dim stmBinaryStream As New Stream
    
    strCommentsHeader = Chr(3) & "vorbis"
    strCommentsHeader = strCommentsHeader & GetBinaryInt(Len(strVendorString))
    strCommentsHeader = strCommentsHeader & strVendorString
    strCommentsHeader = strCommentsHeader & GetBinaryInt(UBound(strCommentsFields))
    For lngCommentCount = 1 To UBound(strCommentsFields)
        strCommentsHeader = strCommentsHeader & GetBinaryInt(Len(strCommentsFields(lngCommentCount)) + Len(strCommentsValues(lngCommentCount)) + 1)
        strCommentsHeader = strCommentsHeader & UCase(strCommentsFields(lngCommentCount)) & "=" & strCommentsValues(lngCommentCount)
    Next
    strCommentsHeader = strCommentsHeader & Chr(1)
    lngLenStrCommentsHeader = Len(strCommentsHeader)
    strTwoHeaders = strCommentsHeader & strSetupHeader
    strHeaderPage = ""
    lngPageCount = 0
    Do While Len(strTwoHeaders) > 0
        lngPageCount = lngPageCount + 1
        strHeaderPageTop = "OggS" & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0) & Chr(0)
        strHeaderPageTop = strHeaderPageTop & GetBinaryInt(lngStreamSerialNumber)
        strHeaderPageTop = strHeaderPageTop & GetBinaryInt(lngPageCount)
        intPageSegments = Int((Len(strTwoHeaders) - lngLenStrCommentsHeader) / 255) + (Int((lngLenStrCommentsHeader + 254) / 255)) + 1
        strHeaderPageBottom = Chr(intPageSegments)
        If intPageSegments > 255 Then intPageSegments = 255
        strPageData = ""
        For intSegmentCount = 1 To intPageSegments
            If lngLenStrCommentsHeader >= 255 Then
                strHeaderPageBottom = strHeaderPageBottom & Chr(255)
                strPageData = strPageData & Mid(strTwoHeaders, 1, 255)
                strTwoHeaders = Mid(strTwoHeaders, 256, Len(strTwoHeaders) - 255)
                lngLenStrCommentsHeader = lngLenStrCommentsHeader - 255
            ElseIf lngLenStrCommentsHeader > 0 Then
                strHeaderPageBottom = strHeaderPageBottom & Chr(lngLenStrCommentsHeader)
                strPageData = strPageData & Mid(strTwoHeaders, 1, lngLenStrCommentsHeader)
                strTwoHeaders = Mid(strTwoHeaders, lngLenStrCommentsHeader + 1, Len(strTwoHeaders) - lngLenStrCommentsHeader)
                lngLenStrCommentsHeader = 0
            ElseIf Len(strTwoHeaders) >= 255 Then
                strHeaderPageBottom = strHeaderPageBottom & Chr(255)
                strPageData = strPageData & Mid(strTwoHeaders, 1, 255)
                strTwoHeaders = Mid(strTwoHeaders, 256, Len(strTwoHeaders) - 255)
            Else
                strHeaderPageBottom = strHeaderPageBottom & Chr(Len(strTwoHeaders))
                strPageData = strPageData & strTwoHeaders
                strTwoHeaders = ""
            End If
        Next
        strHeaderPageBottom = strHeaderPageBottom & strPageData
        strHeaderPage = strHeaderPage & strHeaderPageTop & GetBinaryInt(crc.GetCRC32(strHeaderPageTop & Chr(0) & Chr(0) & Chr(0) & Chr(0) & strHeaderPageBottom)) & strHeaderPageBottom
        Set crc = Nothing
    Loop
    bytHeaderPage = Stream_StringToBinary(strHeaderPage)
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    stmBinaryStream.LoadFromFile strOGGFilename
    bytFirstHeaderPage = stmBinaryStream.Read(58)
    bytNoData = stmBinaryStream.Read(lngAudioPositionStart - 58)
    bytAudioPages = stmBinaryStream.Read(lngFileSize - (lngAudioPositionStart - 58))
    stmBinaryStream.Close
    Set stmBinaryStream = CreateObject("ADODB.Stream")
    stmBinaryStream.Type = adTypeBinary
    stmBinaryStream.Open
    stmBinaryStream.Write bytFirstHeaderPage
    stmBinaryStream.Write bytHeaderPage
    stmBinaryStream.Write bytAudioPages
    stmBinaryStream.SaveToFile strOGGFilename, adSaveCreateOverWrite
    stmBinaryStream.Close
    
    Set stmBinaryStream = Nothing
    Set crc = Nothing
End Sub

'ToBin - Converts a string to a string of its binary representation.
Private Function ToBin(strIn As String) As String
    Dim intStrLen As Integer, intStrVal As Integer, intLenCount As Integer, intBitCount As Integer
    
    If Len(strIn) = 0 Then
        ToBin = ""
        Exit Function
    End If
    intStrLen = Len(strIn) * 8
    ToBin = ""
    For intLenCount = Len(strIn) To 1 Step -1
        intStrVal = Asc(Mid(strIn, intLenCount, 1))
        For intBitCount = 1 To 8
            ToBin = CStr(intStrVal Mod 2) & ToBin
            intStrVal = intStrVal \ 2
        Next
    Next
End Function

'GetBinaryInt - Converts an integer to 4 byte integer.
Private Function GetBinaryInt(lngNumber As Long) As String
    Dim dblNumber As Double
    
    dblNumber = lngNumber
    If dblNumber < 0 Then
        dblNumber = dblNumber + 2147483648#
        GetBinaryInt = Chr(Int(dblNumber / 16777216) + 128)
    Else
        GetBinaryInt = Chr(Int(dblNumber / 16777216))
    End If
    dblNumber = dblNumber Mod 16777216
    GetBinaryInt = Chr(Int(dblNumber / 65536)) & GetBinaryInt
    dblNumber = dblNumber Mod 65536
    GetBinaryInt = Chr(Int(dblNumber / 256)) & GetBinaryInt
    dblNumber = dblNumber Mod 256
    GetBinaryInt = Chr(dblNumber) & GetBinaryInt
End Function

'Stream_StringToBinary Function
'2003 Antonin Foller, http://www.pstruh.cz
'Text - string parameter To convert To binary data
Private Function Stream_StringToBinary(strIn As String) As Byte()
    'Create Stream object
    Dim stmBinaryStream As New Stream

    'Specify stream type - we want To save text/string data.
    stmBinaryStream.Type = adTypeText

    'Specify charset For the source text (unicode) data.
    stmBinaryStream.Charset = "windows-1252"

    'Open the stream And write text/string data To the object
    stmBinaryStream.Open
    stmBinaryStream.WriteText strIn

    'Change stream type To binary
    stmBinaryStream.Position = 0
    stmBinaryStream.Type = adTypeBinary

    'Ignore first two bytes - sign of
    stmBinaryStream.Position = 0

    'Open the stream And get binary data from the object
    Stream_StringToBinary = stmBinaryStream.Read
    
    Set stmBinaryStream = Nothing
End Function
